;****************************************************************************
; Program		: 7 Segment 02
; Description	: 7 Segment HEX Stop Watch < P3.2 Clear, P3.3 Start/Stop >
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in PORT1 mode
;
; Filename		: E1302.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DSP1			BIT		P1.2	; 7 Segment DSP1 enable (Active Low : Level)
DSP2			BIT		P1.3	; 7 Segment DSP2 enable (Active Low : Level)

CLEAR_SW		BIT		P3.2	; Interrupt 0 PIN S2
START_STOP_SW	BIT		P3.3	; Interrupt 1 PIN S3

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02FH	; User FLAG

START			BIT		FLAG.0	; Define START as bit
KEYPRESSED		BIT		FLAG.1	; Define KEYPRESS as bit

DSP1_BUFFER		EQU		030H	; For keep DSP1 Data
DSP2_BUFFER		EQU		031H	; For keep DSP2 Data
DSP_CNT			EQU		032H	; Display Counter Register

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4
		
MAIN:			MOV		DSP_CNT,#0		; Clear Display Counter
				MOV		FLAG,#0			; Clear All FLAG

LOOP:			MOV		A,DSP_CNT		; Move Display Counter to ACC.
				MOV		B,#16			; Move Divisor to Register B
				DIV		AB				; Divider
				MOV		DSP2_BUFFER,A	; Store NUM
				MOV		DSP1_BUFFER,B	; Store DEN

				MOV		R4,#5			; Set loop 5 times
SCAN_DSP_LOOP:	MOV		A,DSP1_BUFFER	; Restore DSP1 to Display
				MOV		DPTR,#DSP_NUM0	; Move DIGIT Start Pointer
				MOVC	A,@A+DPTR		; Get ROM Data from Pointer+ACC.
				MOV		P0,A			; Out ACC. to DATABUS
				CLR		DSP1			; Enable DSP1
				ACALL	DELAY_1ms		; Delay
				SETB	DSP1			; Disable DSP1

				MOV		A,DSP2_BUFFER	; Restore DSP2 to Display
				MOV		DPTR,#DSP_NUM0	; Move DIGIT Start Pointer
				MOVC	A,@A+DPTR		; Get ROM Data from Pointer+ACC.
				MOV		P0,A			; Out ACC. to DATABUS
				CLR		DSP2			; Enable DSP2
				ACALL	DELAY_1ms		; Delay
				SETB	DSP2			; Disable DSP2
				DJNZ	R4,SCAN_DSP_LOOP; Do until 5 times

				JB		KEYPRESSED,NEXT	; Check Flag (S3 still pressed?)
										; Key S3 already released
				SETB	KEYPRESSED		; Set Flag KEYPRESSED
				JNB		START,TIMER_STOP; Check timer status
										; Start now
				INC		DSP_CNT			; Start => increase counter
TIMER_STOP:								; Stop  => not change counter value

				JB		START_STOP_SW,NEXT	; Check S3 pressed?
											; Key S3 pressed
				JNB		START,START_TIMER	; Start
											; Now Start Timer
				CLR		START				; then Stop Timer
				AJMP	NEXT				; Jump to next
START_TIMER:								; Now Stop Timer Status
				SETB	START				; then Start Timer

NEXT:			JNB		START_STOP_SW,CLEAR_CHK	; Check S3 pressed?
				CLR		KEYPRESSED			; Clear Flag KEYPRESSED
CLEAR_CHK:		JB		CLEAR_SW,EXIT		; Check S2 pressed?
											; Key S2 pressed
				MOV		DSP_CNT,#0			; Clear Timer Counter
EXIT:			AJMP	LOOP				; Jump to LOOP

;----------------------------------------------------------------------------
; Dummy Delay time 1m,10ms,1s
;----------------------------------------------------------------------------
DELAY_1ms:		MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_1ms_1:	NOP
				NOP
				DJNZ	R6,DELAY_1ms_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;				Segment .GFEDCBA
DSP_NUM0:		DB		00111111B
DSP_NUM1:		DB		00000110B
DSP_NUM2:		DB		01011011B
DSP_NUM3:		DB		01001111B
DSP_NUM4:		DB		01100110B
DSP_NUM5:		DB		01101101B
DSP_NUM6:		DB		01111101B
DSP_NUM7:		DB		00000111B
DSP_NUM8:		DB		01111111B
DSP_NUM9:		DB		01101111B
DSP_NUMA:		DB		01110111B
DSP_NUMB:		DB		01111100B
DSP_NUMC:		DB		00111001B
DSP_NUMD:		DB		01011110B
DSP_NUME:		DB		01111001B
DSP_NUMF:		DB		01110001B
