;****************************************************************************
; Program		: LCD Display 01
; Description	: LCD Display Show , 16 Char from ROM , Shift Display
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
; Filename		: E1501.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
LCD_EN			BIT		P3.6	; LCD Module Enable (Active High : Level)
LCD_RS			BIT		P3.7	; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR		EQU		030H	; For keep LCD Address
LCD_DATA		EQU		031H	; For keep LCD Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			ACALL	INIT_LCD		; Call LCD Initial subroutine

LOOP:			MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH (Increase automatic)
				MOV		DPTR,#TITLE_3	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 10H-1FH (Increase automatic)

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_2	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH (Increase automatic)
				MOV		DPTR,#TITLE_4	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 50H-5FH (Increase automatic)

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s

				MOV		R4,#16			; Set Loop 8 times
LOOP_LCD_L_SHF: ACALL	LCD_LSHF		; Left Shift LCD Display
				ACALL	DELAY_100ms		; Delay
				ACALL	DELAY_100ms
				DJNZ	R4,LOOP_LCD_L_SHF ; Do until 8 times

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_5	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH (Increase automatic)

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_6	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH (Increase automatic)

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s

				MOV		R4,#16			; Set Loop 8 times
LOOP_LCD_R_SHF: ACALL	LCD_RSHF		; Right Shift LCD Display
				ACALL	DELAY_100ms		; Delay
				ACALL	DELAY_100ms
				DJNZ	R4,LOOP_LCD_R_SHF ; Do until 8 times

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s

				ACALL	LCD_CLR			; Clear LCD Display

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s

				MOV		LCD_DATA,#'H'	; Write Character 'H'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s

				MOV		LCD_DATA,#'e'	; Write Character 'e'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'l'	; Write Character 'l'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'l'	; Write Character 'l'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'o'	; Write Character 'o'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#' '	; Write Space
				ACALL	WRCHAR_LCD
				MOV		LCD_DATA,#'W'	; Write Character 'W'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'o'	; Write Character 'o'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'r'	; Write Character 'r'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'l'	; Write Character 'l'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#'d'	; Write Character 'd'
				ACALL	WRCHAR_LCD
				ACALL	LCD_BLINK		; Blink Cursor
				ACALL	DELAY_1s		; Delay

				MOV		LCD_DATA,#' '	; Write Space
				ACALL	WRCHAR_LCD
				MOV		LCD_DATA,#'I'	; Write Character 'I'
				ACALL	WRCHAR_LCD
				MOV		LCD_DATA,#'N'	; Write Character 'N'
				ACALL	WRCHAR_LCD
				MOV		LCD_DATA,#'E'	; Write Character 'E'
				ACALL	WRCHAR_LCD
				MOV		LCD_DATA,#'X'	; Write Character 'X'
				ACALL	WRCHAR_LCD
				ACALL	DELAY_1s		; Delay

				ACALL	LCD_OFF			; Off Display
				ACALL	DELAY_1s		; Delay

				ACALL	LCD_ON			; On Display
				ACALL	DELAY_1s		; Delay

				ACALL	LCD_OFF			; Off Display
				ACALL	DELAY_1s		; Delay

				ACALL	LCD_ON			; On Display
				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s

				AJMP	LOOP			; Jump to loop

;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:		ACALL	DELAY_100ms		; Delay
				CLR		LCD_RS			; Clear LCD_RS Pin

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	DELAY_10ms		; Delay

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_OFF			; Display Off

				ACALL	LCD_CLR			; Clear Display

				MOV		P0,#00000110B	; Entry Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_HOME		; Return Home Display

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000001B	; Display Clear
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000010B	; Return Home
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001000B	; Display Off
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:		SETB	LCD_EN			; Pulse Clock to LCD_EN
				ACALL	LCD_DELAY
				CLR		LCD_EN
				ACALL	LCD_DELAY
				RET

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:			CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001100B	; Display On
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001111B	; Display Cursor and Blink
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00011000B	; Left Shift Display
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00011100B	; Right Shift Display
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:		LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		A,LCD_ADDR		; Move LCD_ADDR to ACC.
				SETB	ACC.7			; Set bit ACC.7
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:		SETB	LCD_RS			; Set LCD_RS Pin
				MOV		P0,LCD_DATA		; Move LCD_DATA to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:		MOV		R0,#0			; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Move data from @DPTR to ACC.
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase loop counter
				CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m, 100m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:		MOV		R7,#002			; Do 2 times
LCD_DELAY_1:	MOV		R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
				NOP
				DJNZ	R6,LCD_DELAY_2
				DJNZ	R7,LCD_DELAY_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;						 0123456789ABCDEF
TITLE_1:		DB		'LCD Module Show '
TITLE_2:		DB		' 16x2 character '

TITLE_3:		DB		'Test Running ...'
TITLE_4:		DB		' Left & Right   '

TITLE_5:		DB		'Write Character '
TITLE_6:		DB		'  on sequence   '