;****************************************************************************
; Program		: Real Time Clock 01
; Description	: Real Time Clock , S2 Preset Date , S3 Preset Time
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in PORT1 mode
;
; Filename		: E2001.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
SDA				BIT		P1.0		; SDA I2C Bus
SCL				BIT		P1.1		; SCL I2C Bus

KPAD_ROW0		BIT		P2.0		; Keypad Input Row 0
KPAD_ROW1		BIT		P2.1		; Keypad Input Row 1
KPAD_ROW2		BIT		P2.2		; Keypad Input Row 2
KPAD_ROW3		BIT		P2.3		; Keypad Input Row 3
KPAD_COL2		BIT		P2.4		; Keypad Output Column 2
KPAD_COL1		BIT		P2.5		; Keypad Output Column 1
KPAD_COL0		BIT		P2.6		; Keypad Output Column 0

SET_DATE_SW		BIT		P3.2		; Interrupt 0 PIN S2
SET_TIME_SW		BIT		P3.3		; Interrupt 1 PIN S3
LCD_EN			BIT		P3.6		; LCD Module Enable (Active High : Level)
LCD_RS			BIT		P3.7		; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02FH		; User FLAG
I2C_ACK			BIT		FLAG.0		; Define I2C Acknowledge as bit

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR		EQU		030H		; For keep LCD Address
LCD_DATA		EQU		031H		; For keep LCD Data
LCD_PTR			EQU		032H		; For keep LCD 3 Cha.r Pointer
I2C_ADDR		EQU		033H		; For keep I2C Address
I2C_DATA		EQU		034H		; For keep I2C Data
KPAD_DATA		EQU		035H		; For keep Keypad Data
BUFFER			EQU		036H		; Buffer

SECONDS			EQU		040H		; For keep Seconds
MINUTES			EQU		041H		; For keep Minutes
HOURS			EQU		042H		; For keep Hours
DAY				EQU		043H		; For keep Day
DATE			EQU		044H		; For keep Date
MONTH			EQU		045H		; For keep Month
YEAR			EQU		046H		; For keep Year
CONTROL			EQU		047H		; For keep Control Byte

;----------------------------------------------------------------------------
; Define I2C Slave Address
;----------------------------------------------------------------------------
RTC_ID			EQU		11010000B 	; RTC Slave Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			ACALL	INIT_LCD		; Call LCD Initial subroutine

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

;				ACALL	DELAY_1s		; Delay
;				ACALL	DELAY_1s

LOOP:			ACALL	RTC_RD			; Read RTC

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;

				MOV		LCD_DATA,DATE	; Write Date to LCD
				ACALL	BCD2LCD			; Write from RTC BCD Data

				MOV		LCD_DATA,#'-'	; Write space to LCD
				ACALL	WRCHAR_LCD		;

				MOV		A,MONTH			; Convert Month from BCD to HEX
				CJNE	A,#010H,WR_CHK_MONTH_1 ;
				MOV		A,#00AH			; October => 0AH
				AJMP	WRITE_MONTH_NX	;
WR_CHK_MONTH_1: CJNE	A,#011H,WR_CHK_MONTH_2 ;
				MOV		A,#00BH			; November => 0BH
				AJMP	WRITE_MONTH_NX	;
WR_CHK_MONTH_2: CJNE	A,#012H,WRITE_MONTH_NX ;
				MOV		A,#00CH			; December => 0CH
WRITE_MONTH_NX: MOV		LCD_PTR,A		; Set 3 Char. Pointer
				MOV		DPTR,#MONTH_JAN ; Set Start Pointer
				ACALL	WR3CHAR_LCD		; Write 3 Char. (Month) to LCD

				MOV		LCD_DATA,#'-'	; Write space to LCD
				ACALL	WRCHAR_LCD		;

				MOV		LCD_DATA,YEAR	; Write Year to LCD
				ACALL	BCD2LCD			; Write from RTC BCD Data

				MOV		LCD_DATA,#' '	; Write space to LCD
				ACALL	WRCHAR_LCD		;

				MOV		LCD_DATA,#' '	; Write space to LCD
				ACALL	WRCHAR_LCD		;

				MOV		A,HOURS			; Get Hour
				ANL		A,#00110000B	; Get x10 Digit
				JZ		WRITE_TIME_HN	; Check x10 Digit = 0 ?
				SWAP	A				; If not => Write to LCD
				ADD		A,#030H			; Convert to ASCII
				AJMP	WRITE_TIME_HH	;
WRITE_TIME_HN:	MOV		A,#' '			; x10 = 0 then Write space
WRITE_TIME_HH:	MOV		LCD_DATA,A		; Write to LCD
				ACALL	WRCHAR_LCD		;

				MOV		A,HOURS			; Write x1 Digit to LCD
				ANL		A,#00001111B	;
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		;
				ACALL	WRCHAR_LCD		;

				MOV		A,SECONDS		; Check Second = Odd Number ?
				ANL		A,#001H			;
				JNZ		WRITE_SPACE		; Even => Write space
				MOV		LCD_DATA,#':'	; Odd => Write ':'
				ACALL	WRCHAR_LCD		;
				AJMP	WRITE_MINUTES	;
WRITE_SPACE:	MOV		LCD_DATA,#' '	;
				ACALL	WRCHAR_LCD		;

WRITE_MINUTES:	MOV		LCD_DATA,MINUTES ; Write Minutes to LCD
				ACALL	BCD2LCD			; Write from RTC BCD Data

				JNB		SET_DATE_SW,SET_DATE	; Check S2 Pressed? => Set Date
				JNB		SET_TIME_SW,SET_TIME_1	; Check S3 Pressed? => Set Time
				AJMP	LOOP			; Jump to loop

SET_TIME_1:		AJMP	SET_TIME		; Jump to Set Time

SET_DATE:		MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_SET_DATE ; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

				MOV		LCD_ADDR,#048H	; Set Address 48H
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		DATE,A			; Keep Date Data

				MOV		LCD_ADDR,#04BH	; Set Address 4BH
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		MONTH,A			; Keep Month Data

				MOV		LCD_ADDR,#04EH	; Set Address 4EH
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		YEAR,A			; Keep Year Data

				ACALL	RTC_WR			; Write RTC
				AJMP	LOOP			; Jump to loop

SET_TIME:		MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_SET_TIME ; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

				MOV		LCD_ADDR,#048H	; Set Address 48H
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		HOURS,A			; Keep Hours Data

				MOV		LCD_ADDR,#04BH	; Set Address 4BH
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		MINUTES,A		; Keep Minutes Data

				MOV		LCD_ADDR,#04EH	; Set Address 4EH
				ACALL	SET_ADDR_LCD	;
				ACALL	LCD_BLINK		; Blink LCD

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER,KPAD_DATA ; Copy in BUFFER
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	LCD_BLINK		; Blink LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	WAIT_KEYPRESSED ; Wait until Keypad pressed
				MOV		BUFFER+1,KPAD_DATA ; Copy in BUFFER+1
				ACALL	KPAD2LCD		; Show Keypad on LCD
				ACALL	WAIT_KEY		; Wait until Keypad depressed

				ACALL	BUFFER2ACC		; Get 2 buffer to ACC
				MOV		SECONDS,A		; Keep Seconds Data

				ACALL	RTC_WR			; Write RTC
				AJMP	LOOP			; Jump to loop


;----------------------------------------------------------------------------
; BUFFER to ACC
; I/P:		BUFFER,BUFFER+1
; O/P:		ACC
;----------------------------------------------------------------------------
BUFFER2ACC:		MOV		A,BUFFER		; Restore BUFFER
				ANL		A,#00FH		; Get lower 4 bit
				SWAP	A		; Swap nibble
				MOV		B,A		; Move to B
				MOV		A,BUFFER+1		; Restore BUFFER+1
				ANL		A,#00FH		; Get lower 4 bit
				ADD		A,B		; Add BUFFER to make RTC BCD
				RET

;----------------------------------------------------------------------------
; KEYPAD to show LCD
; I/P:		KPAD_DATA
;----------------------------------------------------------------------------
KPAD2LCD:		MOV		A,KPAD_DATA		; Get Keypad Data
				ADD		A,#030H		; Convert to ASCII
				MOV		LCD_DATA,A		; Write to lCD
				ACALL	WRCHAR_LCD		;
				RET

;----------------------------------------------------------------------------
; BCD Code to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
BCD2LCD:		PUSH	ACC				; Push ACC. To Stack
				PUSH	B				; Push B to Stack
				MOV		A,LCD_DATA		; Get input data value
				MOV		B,A				; Copy to B
				ANL		A,#11110000B	; Get higher 4 bit
				SWAP	A				; Swap nibble
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		; Write LCD
				ACALL	WRCHAR_LCD		;
				MOV		A,B				; Restore value
				ANL		A,#00001111B	; Get lower 4 bit
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		; Write LCD
				ACALL	WRCHAR_LCD		;
				POP		B				; Pop B from Stack
				POP		ACC				; Pop ACC. from Stack
				RET						; Return

;----------------------------------------------------------------------------
; Wait keypad depressed
;----------------------------------------------------------------------------
WAIT_KEY:		MOV		A,P2			; Move P2 to ACC.
				ANL		A,#00FH			; Get only Lower 4 bit
				CJNE	A,#00FH,WAIT_KEY ; All bit are high ?
				RET

;----------------------------------------------------------------------------
; Wait keypad pressed 0-9 Only
;----------------------------------------------------------------------------
WAIT_KEYPRESSED: ACALL	GET_KPAD		; Check keypad Data
				MOV		A,KPAD_DATA		;
				CJNE	A,#0,CHK_KEY_NEXT ; Check keypad pressed?
				AJMP	WAIT_KEYPRESSED ; Loop until keypad pressed
CHK_KEY_NEXT:	CJNE	A,#10,CHK_KEY_0 ; Check key *
				AJMP	WAIT_KEYPRESSED ; If key * then loop wait
CHK_KEY_0:		CJNE	A,#11,CHK_VALID_KEY ; Check key 0
				MOV		KPAD_DATA,#0	; Replace KPAD_DATA = 0
				RET						; Return
CHK_VALID_KEY:	JNC		WAIT_KEYPRESSED ; If KPAD_DATA > 11 then loop wait
				RET						; Return

;----------------------------------------------------------------------------
; Keypad Scan key Subroutine
;----------------------------------------------------------------------------
GET_KPAD:		MOV		P2,#0FFH		; Pull P2 to High
				MOV		KPAD_DATA,#0	; Clear Keypad Data

CHK_COL0:		CLR		KPAD_COL0		; Begin Scan Column 0
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL0_DETECT ; Check All rows '1'?
				AJMP	CHK_COL1		; All rows '1' => check next column

COL0_DETECT:	MOV		KPAD_DATA,#01	; Initial KPAD_DATA = 1
				AJMP	GET_ROW			; Jump to get row value

CHK_COL1:		SETB	KPAD_COL0		; Stop Scan Column 0
				CLR		KPAD_COL1		; Begin Scan Column 1
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL1_DETECT ; Check All rows '1'?
				AJMP	CHK_COL2		; All rows '1' => check next column

COL1_DETECT:	MOV		KPAD_DATA,#02	; Initial KPAD_DATA = 2
				AJMP	GET_ROW			; Jump to get row value

CHK_COL2:		SETB	KPAD_COL1		; Stop Scan Column 1
				CLR		KPAD_COL2		; Begin Scan Column 2
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL2_DETECT ; Check All rows '1'?
				RET						; All rows '1' => return

COL2_DETECT:	MOV		KPAD_DATA,#03	; Initial KPAD_DATA = 2

GET_ROW:		CLR		KPAD_COL0		; Enable all Column to find Crosspoint
				CLR		KPAD_COL1		;
				CLR		KPAD_COL2		;

				JB		KPAD_ROW0,CHK_ROW1 ; Check Row 0 Detect?
				RET						; Row 0 Detect => return

CHK_ROW1:		JB		KPAD_ROW1,CHK_ROW2 ; Check Row 2 Detect?
				MOV		A,KPAD_DATA		; Add 3 with KPAD_DATA
				ADD		A,#3			;
				MOV		KPAD_DATA,A		;
				RET						; Return

CHK_ROW2:		JB		KPAD_ROW2,CHK_ROW3 ; Check Row 2 Detect?
				MOV		A,KPAD_DATA		; Add 6 with KPAD_DATA
				ADD		A,#6			;
				MOV		KPAD_DATA,A		;
				RET						; Return

CHK_ROW3:		MOV		A,KPAD_DATA		; Add 9 with KPAD_DATA
				ADD		A,#9			;
				MOV		KPAD_DATA,A		;
				RET						; Return

;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:		ACALL	DELAY_100ms		; Delay
				CLR		LCD_RS			; Clear LCD_RS Pin

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	DELAY_10ms		; Delay

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_OFF			; Display Off

				ACALL	LCD_CLR			; Clear Display

				MOV		P0,#00000110B	; Entry Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_HOME		; Return Home Display

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000001B	; Display Clear
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000010B	; Return Home
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001000B	; Display Off
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:		SETB	LCD_EN			; Pulse Clock to LCD_EN
				ACALL	LCD_DELAY
				CLR		LCD_EN
				ACALL	LCD_DELAY
				RET

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:			CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001100B	; Display On
				ACALL	LCD_CLK
				RET

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001111B	; Display Cursor and Blink
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:		LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		A,LCD_ADDR		; Move LCD_ADDR to ACC.
				SETB	ACC.7			; Set bit ACC.7
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:		SETB	LCD_RS			; Set LCD_RS Pin
				MOV		P0,LCD_DATA		; Move LCD_DATA to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:		MOV		R0,#0			; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Move data from @DPTR to ACC.
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase loop counter
				CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write 3 Character from ROM
; I/P:		LCD_PTR
;----------------------------------------------------------------------------
WR3CHAR_LCD:	MOV		R0,#0			; Clear Loop Counter
				MOV		A,LCD_PTR		; Get LCD Pointer
				DEC		A				; Decrease Value
				MOV		B,#3			;
				MUL		AB				; Multiply by 3
WR3CHAR_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase Loop Counter

				MOV		A,LCD_PTR		; Restore LCD Pointer
				DEC		A				; Decrease Value
				MOV		B,#3			;
				MUL		AB				; Multiply by 3

				CJNE	R0,#3,WR3CHAR_LCD_1 ; Do until 3 times
				LCALL	LCD_ON			; Show LCD
				RET						; Return

;----------------------------------------------------------------------------
; I2C RTC Read
;----------------------------------------------------------------------------
RTC_RD:			MOV		I2C_ADDR,#RTC_ID ; Set RTC as I2C Write Slave
				LCALL	I2C_SLAVE		; Connect Slave
		
				MOV		I2C_DATA,#000H	; Set Slave Address 00H
				LCALL	I2C_DATA_WR		; Write Data to Slave

				MOV		I2C_ADDR,#RTC_ID+1 ; Set RTC as I2C Read Slave
				LCALL	I2C_SLAVE		; Connect Slave
				LCALL	I2C_DATA_RD		; Read Data from Slave

				MOV		SECONDS,I2C_DATA ; Read Data to SECONDS
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		MINUTES,I2C_DATA ; Read Data to MINUTES
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		HOURS,I2C_DATA	; Read Data to HOURS
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		DAY,I2C_DATA	; Read Data to DAY
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		DATE,I2C_DATA	; Read Data to DATE
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		MONTH,I2C_DATA	; Read Data to MONTH
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		YEAR,I2C_DATA	; Read Data to YEAR
				LCALL	I2C_ACK_BIT		; Send Acknowledge

				LCALL	I2C_DATA_RD		; Read Data from Slave
				MOV		CONTROL,I2C_DATA ; Read Data to CONTROL
				LCALL	I2C_NACK_BIT	; Send Not Acknowledge

				LCALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C RTC Write
;----------------------------------------------------------------------------
RTC_WR:			MOV		I2C_ADDR,#RTC_ID ; Set RTC as I2C Write Slave
				LCALL	I2C_SLAVE		; Connect Slave

				MOV		I2C_DATA,#000H	; Set Slave Address 00H
				LCALL	I2C_DATA_WR		; Write Data to Slave

				MOV		I2C_DATA,SECONDS ; Write SECONDS to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,MINUTES ; Write MINUTES to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,HOURS	; Write HOURS to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,DAY	; Write DAY to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,DATE	; Write DATE to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,MONTH	; Write MONTH to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,YEAR	; Write YEAR to RTC
				LCALL	I2C_DATA_WR		;

				MOV		I2C_DATA,CONTROL ; Write CONTROL to RTC
				LCALL	I2C_DATA_WR		;

				LCALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Write
; I/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_WR:	PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_DATA		; Get Data
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_WR_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_DATA_WR_1	; Do until 8 times
				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_DATA_WR_2	; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK. bit
I2C_DATA_WR_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Read
; O/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_RD:	PUSH	ACC				; Push ACC.
				CLR		A				; Clear ACC.
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_RD_1:	ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				MOV		C,SDA			; Get SDA to Carry Flag
				RLC		A				; Rotate ACC. to Left with Carry
				CLR		SCL				; Clear SCL
				DJNZ	R5,I2C_DATA_RD_1; Do until 8 times
				MOV		I2C_DATA,A		; Move Data to I2C_DATA
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Slave Connect
; I/P:		I2C_ADDR
; O/P Flag:	I2C_ACK
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_SLAVE:		PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_ADDR		; Get Slave Address
				ACALL	I2C_START		; Send Start Condition
				MOV		R5,#008			; Set loop 8 times
I2C_SLAVE_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_SLAVE_1	; Do until 8 times

				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_SLAVE_2 ; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK.
I2C_SLAVE_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Start Condition
;----------------------------------------------------------------------------
I2C_START:		JNB		SCL,I2C_START_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_START_1:	SETB	SDA				; Set SDA
				SETB	SCL				; Set SCL

				ACALL	I2C_DELAY		; Delay
				CLR		SDA				; Clear SDA during SCL set
				ACALL	I2C_DELAY		; Delay
				CLR		SCL				; Clear SCL
				RET						; Return

;----------------------------------------------------------------------------
; I2C Stop Condition
;----------------------------------------------------------------------------
I2C_STOP:		JNB		SCL,I2C_STOP_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_STOP_1:		CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				SETB	SDA				; Set SDA during SCL set
				RET						; Return

;----------------------------------------------------------------------------
; I2C Clock
;----------------------------------------------------------------------------
I2C_CLK:		ACALL	I2C_DELAY		; Pulse SCL
				SETB	SCL				;
				ACALL	I2C_DELAY		;
				CLR		SCL				;
				RET						; Return

;----------------------------------------------------------------------------
; I2C Acknowledge
;----------------------------------------------------------------------------
I2C_ACK_BIT:	CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SDA
				RET						; Return

;----------------------------------------------------------------------------
; I2C Not Acknowledge
;----------------------------------------------------------------------------
I2C_NACK_BIT:	SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SCL
				RET						; Return

;----------------------------------------------------------------------------
; Dummy Delay time I2C_DELAY, LCD_DELAY, 10m, 100m, 1s
;----------------------------------------------------------------------------
I2C_DELAY:		MOV		R6,#00CH		; Each loop = 50 us
I2C_DELAY_1:	NOP
				NOP
				DJNZ	R6,I2C_DELAY_1
				RET

LCD_DELAY:		MOV		R7,#002			; Do 2 times
LCD_DELAY_1:	MOV		R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
				NOP
				DJNZ	R6,LCD_DELAY_2
				DJNZ	R7,LCD_DELAY_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;						 0123456789ABCDEF
TITLE_1:		DB		'Real Time Clock '
SCR_SET_DATE:	DB		'Date :  dd/mm/yy'
SCR_SET_TIME:	DB		'Time :  hh/mm/ss'

MONTH_JAN:		DB		'Jan'
MONTH_FEB:		DB		'Feb'
MONTH_MAR:		DB		'Mar'
MONTH_APR:		DB		'Apr'
MONTH_MAY:		DB		'May'
MONTH_JUN:		DB		'Jun'
MONTH_JUL:		DB		'Jul'
MONTH_AUG:		DB		'Aug'
MONTH_SEP:		DB		'Sep'
MONTH_OCT:		DB		'Oct'
MONTH_NOV:		DB		'Nov'
MONTH_DEC:		DB		'Dec'