;****************************************************************************
; Program		: Counter 01
; Description	: Frequency Counter
;				: Digit Display 0-255
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
; Filename		: E2401.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
T0_PIN			BIT		P3.4		; Timer/Counter 0 PIN
LCD_EN			BIT		P3.6		; LCD Module Enable (Active High : Level)
LCD_RS			BIT		P3.7		; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02FH		; User FLAG
OVER			BIT		FLAG.0		; Define OVER as bit

LCD_ADDR		EQU		030H		; For keep LCD Address
LCD_DATA		EQU		031H		; For keep LCD Data

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector   
				 
				AJMP	INITIAL    
				
				ORG		000BH			; TF0 Vector
				SETB	OVER			; Set bit OVER Range
				RETI					; Retrun interrupt

			
; Initial all port for NX-51 V.2.0
INITIAL:		MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			ACALL	INIT_LCD		; Call LCD Initial subroutine

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH (Increase automatic)

MAIN_LOOP:		MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_RUN	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH (Increase automatic)

				MOV		TMOD,#00100110B ; T1 8Bit Auto, C0 8Bit Gate disable

				SETB	EA

FREQ_LOOP:		MOV		TH0,#000H		; Clear Loop Counter
				MOV		TL0,#000H		;

				CLR		OVER			; Clear Over Flag
				SETB	ET0				; Enable Counter 0 INT.
				SETB	TR0				; Start Counter 0

GATE_EN:		MOV		R7,#100			; Set gate time = 1s
GATE_EN_1:		MOV		R6,#10			; Each loop = 10 ms
GATE_EN_2:		MOV		R5,#0E6H		;
GATE_EN_3:		NOP
				NOP
				DJNZ	R5,GATE_EN_3	;
				DJNZ	R6,GATE_EN_2	;
				DJNZ	R7,GATE_EN_1	;
				
				CLR		TR0				; Stop Counter 0
				CLR		ET0				; Disable Counter 0 INT.

				JB		OVER,WRITE_OVER ; Check Over Range ?

				MOV		LCD_ADDR,#048H	; Set Address 048H
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,TL0	; Get Counter Value
				ACALL	HEX2LCD			; Write to LCD

				ACALL	DELAY_250ms		; Delay Screen
				AJMP	FREQ_LOOP		; Jump to Read Frequency again

WRITE_OVER:		MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_OVER	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH (Increase automatic)
				ACALL	DELAY_250ms		; Delay Screen

				AJMP	MAIN_LOOP		; Jump to Main loop

;----------------------------------------------------------------------------
; HEX Code to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
HEX2LCD:		PUSH	ACC				; Push ACC.
				MOV		A,LCD_DATA		; Get Data
				MOV		B,#100			;
				DIV		AB				; Divide by 100
				ADD		A,#030H			; Convert to ASCII
				CJNE	A,#030H,HEX2_LCD_NX ; Check x100 = 0 ?
				MOV		A,#' '			; 0 => Write Space
HEX2_LCD_NX:	MOV		LCD_DATA,A		;
				ACALL	WRCHAR_LCD		; Write x100
				MOV		A,B				;
				MOV		B,#10			;
				DIV		AB				; Divide by 10
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		;
				ACALL	WRCHAR_LCD		; Write Lower HEX Code
				MOV		A,B				; Get Remainder x1
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		;
				ACALL	WRCHAR_LCD		; Write Lower HEX Code
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:		ACALL	DELAY_100ms		; Delay
				CLR		LCD_RS			; Clear LCD_RS Pin

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	DELAY_10ms		; Delay

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_OFF			; Display Off

				ACALL	LCD_CLR			; Clear Display

				MOV		P0,#00000110B	; Entry Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_HOME		; Return Home Display

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000001B	; Display Clear
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000010B	; Return Home
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001000B	; Display Off
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:		SETB	LCD_EN			; Pulse Clock to LCD_EN
				ACALL	LCD_DELAY
				CLR		LCD_EN
				ACALL	LCD_DELAY
				RET

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:			CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001100B	; Display On
				ACALL	LCD_CLK
				RET

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:		LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		A,LCD_ADDR		; Move LCD_ADDR to ACC.
				SETB	ACC.7			; Set bit ACC.7
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:		SETB	LCD_RS			; Set LCD_RS Pin
				MOV		P0,LCD_DATA		; Move LCD_DATA to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:		MOV		R0,#0			; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Move data from @DPTR to ACC.
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase loop counter
				CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m, 100m, 250m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:		MOV		R7,#002			; Do 2 times
LCD_DELAY_1:	MOV		R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
				NOP
				DJNZ	R6,LCD_DELAY_2
				DJNZ	R7,LCD_DELAY_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

DELAY_250ms:	MOV		R7,#250			; Do 250 times
DELAY_250ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_250ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_250ms_2
				DJNZ	R7,DELAY_250ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;						 0123456789ABCDEF
TITLE_1:		DB		' Freq. Counter  '

SCR_RUN:		DB		'FREQ :        Hz'
SCR_OVER:		DB		'Input over range'