;****************************************************************************
; Program		: Serial UART 01
; Description	: Serial UART Write Data to PC
;				: using Bit polled check method
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
; Filename		: E2501.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************
;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector

										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		TMOD,#021H		; T1 8Bit Auto, T0 16Bit
				MOV		TH1,#0FDH		; 9600 bps Timer1 Default
				MOV		TL1,#0FDH		;

				SETB	TR1				; Start Timer1

				MOV		SCON,#040H		; Mode1 RX Disable

				MOV		DPTR,#SERIAL_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		DPTR,#HELLO_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				AJMP	$

;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:		CLR		TI				; Clear TI
TX_LOOP:		CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				INC		DPTR			; Increase Pointer
				CJNE	A,#0FFH,TX_CHAR ; Check 0FFH End of Text Char.
				RET						; End => Return
TX_CHAR:		MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				ACALL	DELAY_100ms		; Delay
				AJMP	TX_LOOP			; Jump to TX_LOOP

;----------------------------------------------------------------------------
; Dummy Delay time 100m
;----------------------------------------------------------------------------
DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP
				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
SERIAL_TEXT:	DB		00AH,00DH
				DB		00AH,00DH
				DB		'Serial UART Communication < baudrate 9600bps > ',00AH,00DH
				DB		'for MCS-51 Microcontroller Training System',00AH,00DH
				DB		'Copyright (C) 2000 Innovative Experiment Co.,Ltd.',00AH,00DH
				DB		00AH,00DH,0FFH
				
HELLO_TEXT:		DB		00DH,'Hello.. PC.',00AH,00DH,0FFH