;****************************************************************************
; Program		: Programmable clock out
; Description	: Generate programmable clock out at P1.0
;				: using Timer 2
; For			: NX-51 Version 2.0 (Philips P89C51RD+)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in T2/PCA mode
;
; Filename		: E2701.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define MCS-51 Timer2 Register
; (This section may not define on other assembler)
;----------------------------------------------------------------------------
T2CON			EQU		0C8H	; T2CON Address in SFR
T2MOD			EQU		0C9H	; T2MOD Address in SFR
RCAP2L			EQU		0CAH	; RCAP2L Address in SFR
RCAP2H			EQU		0CBH	; RCAP2H Address in SFR
TL2				EQU		0CCH	; TL2 Address in SFR
TH2				EQU		0CDH	; TH2 Address in SFR

;----------------------------------------------------------------------------
; Define MCS-51 Timer2 bit addressable
; (This section may not define on other assembler)
;----------------------------------------------------------------------------
CP_RL2			BIT		0C8H	; CP/RL2 bit in T2CON
C_T2			BIT		0C9H	; C_T2 bit in T2CON
TR2				BIT		0CAH	; TR2 bit in T2CON
EXEN2			BIT		0CBH	; EXEN2 bit in T2CON
TCLK			BIT		0CCH	; TCLK bit in T2CON
RCLK			BIT		0CDH	; RCLK bit in T2CON
EXF2			BIT		0CEH	; EXF2 bit in T2CON
TF2				BIT		0CFH	; TF2 bit in T2CON

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector

MAIN:									; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				MOV		T2MOD,#002H		; Set Timer2 mode as T2OE=1 timer2 output enable
				MOV		TH2,#0CAH		; Set reload value for clock out 200Hz
				MOV		TL2,#000H    	;
				MOV		RCAP2H,#0CAH	; Set reload value for clock out 200Hz
				MOV		RCAP2L,#000H    ;
				SETB	TR2				; Start timer2
				AJMP	$