;****************************************************************************
; Program		: Port Output 02
; Description	: LED Running with sound on Piezzo
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1102.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		A,#01010101B	; Initial ACC. Value

LOOP_LED:		MOV		P0,A			; Out ACC. Value to DATABUS
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
				MOV		R4,#200			; Set 200 times loop

LOOP_SOUND:		MOV		A,P0			; Get value from DATABUS to ACC.
				CPL		ACC.5			; Complement ACC. Bit 5
				MOV		P0,A			; Replace.value to DATABUS
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
				ACALL	DELAY_100us		; Delay 100us
				DJNZ	R4,LOOP_SOUND	; Do loop until 200 times

				CPL		A				; Complement ACC.
				ACALL	DELAY_1s		; Delay 1 s
				AJMP	LOOP_LED		; Jump to loop LED

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE		; Driver Latch Enable
				NOP						; NOP
				CLR		DRIVER_LE		; Driver Latch Disable
				NOP
				RET

;----------------------------------------------------------------------------
; Dummy Delay time 100us,10m,1s
;----------------------------------------------------------------------------
DELAY_100us:	MOV		R6,#017H		; Each loop = 100 us
DELAY_100us_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100us_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET
