;****************************************************************************
; Program		: Port Input 01
; Description	: Key Input from S2(P3.2) to "ON" Relay and S3(P3.3) to "OFF"
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1201.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

ON_SW			BIT		P3.2	; Interrupt 0 PIN S2
OFF_SW			BIT		P3.3	; Interr2upt 1 PIN S3

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				SETB	DRIVER_LE		; Enable Driver Latch

MAIN:			JB		ON_SW,CHK_OFF_SW; Check S2 pressed?
				ACALL	DELAY_10ms		; Delay to avoid switch bounce
				JB		ON_SW,CHK_OFF_SW; Repeat check S2 Pressed?
				SETB	P0.4			; On Relay
				JNB		ON_SW,$			; Wait until S2 depressed
				AJMP	MAIN			; Jump to main

CHK_OFF_SW:		JB		OFF_SW,MAIN		; Check S3 pressed?
				ACALL	DELAY_10ms		; Delay to avoid switch bounce
				JB		OFF_SW,MAIN		; Repeat check S3 Pressed?
				CLR		P0.4			; Off Relay
				JNB		OFF_SW,$		; Wait until S3 depressed
				AJMP	MAIN			; Jump to main

;----------------------------------------------------------------------------
; Dummy Delay time 10m
;----------------------------------------------------------------------------
DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET
