;****************************************************************************
; Program		: Port Input 02
; Description	: Key Input from S2(P3.2) Shift LED Left /S3(P3.3) Shift Right
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1202.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

LEFT_SHF		BIT		P3.2	; Interrupt 0 PIN S2
RIGHT_SHF		BIT		P3.3	; Interrupt 1 PIN S3

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		A,#10000000B	; Initial ACC. Value
				MOV		P0,A			; Out ACC. Value to DATABUS
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock

LOOP:			JB		LEFT_SHF,CHK_S3	; Check S2 pressed?
				ACALL	DELAY_10ms		; Delay to avoid switch bounce
				JB		LEFT_SHF,CHK_S3	; Repeat check S2 Pressed?
				RL		A
				MOV		P0,A			; Out ACC. Value to DATABUS
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
				ACALL	DELAY_100ms		; Delay 100ms
				AJMP	LOOP			; Jump to Loop
				
CHK_S3:			JB		RIGHT_SHF,LOOP	; Check S3 pressed?
				ACALL	DELAY_10ms		; Delay to avoid switch bounce
				JB		RIGHT_SHF,LOOP	; Repeat check S3 Pressed?
				RR		A
				MOV		P0,A			; Out ACC. Value to DATABUS
				ACALL	DRIVE_LE_CLK	; Call Drive Latch Enable Clock
				ACALL	DELAY_100ms		; Delay 100ms
				AJMP	LOOP			; Jump to Loop

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE		; Driver Latch Enable
				NOP						; NOP
				CLR		DRIVER_LE		; Driver Latch Disable
				NOP
				RET

;----------------------------------------------------------------------------
; Dummy Delay time 10m,100ms
;----------------------------------------------------------------------------
DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET
