;****************************************************************************
; Program		: LCD Display 03
; Description	: LCD Display , Combination lock (ON Relay)
;				: using Keypad to enter 8 digit compare with ROM
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1503.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

KPAD_ROW0		BIT		P2.0	; Keypad Input Row 0
KPAD_ROW1		BIT		P2.1	; Keypad Input Row 1
KPAD_ROW2		BIT		P2.2	; Keypad Input Row 2
KPAD_ROW3		BIT		P2.3	; Keypad Input Row 3
KPAD_COL2		BIT		P2.4	; Keypad Output Column 2
KPAD_COL1		BIT		P2.5	; Keypad Output Column 1
KPAD_COL0		BIT		P2.6	; Keypad Output Column 0

S2_SW			BIT		P3.2	; Interrupt 0 PIN S2
S3_SW			BIT		P3.3	; Interrupt 1 PIN S3
LCD_EN			BIT		P3.6	; LCD Module Enable (Active High : Level)
LCD_RS			BIT		P3.7	; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02FH	; User FLAG
FAIL			BIT		FLAG.0	; Define FAIL as bit

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR		EQU		030H	; For keep LCD Address
LCD_DATA		EQU		031H	; For keep LCD Data
KPAD_DATA		EQU		032H	; For keep Keypad Data
CG_ADDR			EQU		033H	; For keep CG-RAM Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			ACALL	INIT_LCD		; Call LCD Initial subroutine
				ACALL	LOAD_KEY_CHAR	; Call CG-RAM <Key>,#00H Write Routine

LOOP:			MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH(Increase automatic)

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_2	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH(Increase automatic)

				MOV		LCD_ADDR,#048H	; Set Address 48H
				ACALL	SET_ADDR_LCD		;
				ACALL	LCD_BLINK		; Blink Cursor

				MOV		DPTR,#CODE_ID	; Index Pointer ROM to get Code ID.
				MOV		R4,#8			; Set loop 8 times
				CLR		FAIL			; Clear FAIL Flag  

				ACALL	WAIT_KEY		; Wait keypad depressed

LOOP_KEY_IN:	ACALL	GET_KPAD		; Get kepad Data
				MOV		A,KPAD_DATA		;
				CJNE	A,#000H,NEXT		; Check keypad pressed?
				AJMP	LOOP_KEY_IN		; KPAD_DATA=0 then wait key

NEXT:			ADD		A,#030H			; Add data to ASCII
				CJNE	A,#03AH,CHK_KEY_0; Check Star key (30H+0AH)
				MOV		A,#02AH			; Replace Data to Real ASCII
				AJMP	SHOW_LCD		; Jump to Show LCD

CHK_KEY_0:		CJNE	A,#03BH,CHK_KEY_HASH; Check key 0 (30H+0BH)
				MOV		A,#030H			; Replace Data to Real ASCII
				AJMP	SHOW_LCD		; Jump to Show LCD
CHK_KEY_HASH:	CJNE	A,#03CH,SHOW_LCD; Check Hash key (30H+0CH)
				MOV		A,#023H			; Replace Data to Real ASCII

SHOW_LCD:		MOV		B,A				; Keep Keypad data ASCII to B
				MOV		LCD_DATA,#'*'	; Show * to hidden keypad data
				ACALL	WRCHAR_LCD		;
				ACALL	LCD_BLINK		; Blink Cursor
				CLR		A				; Get Code ID. from ROM
				MOVC	A,@A+DPTR		;
				XRL		A,B				; Compare Keypad Data with ROM
				JZ		ID_PASS			; If identical Char. goto ID_PASS
				SETB	FAIL			; if different => set FAIL Flag
ID_PASS:		INC		DPTR			; Increase Pointer
				ACALL	WAIT_KEY		; Wait until keypad depressed
				ACALL	DELAY_1ms		; Delay
				DJNZ	R4,LOOP_KEY_IN		; Do until 8 times

				JB		FAIL,CODE_FAIL	; FAIL? => goto Code Fail
										; Code Pass => Show Message OK
CODE_PASS:		MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_CODE_PASS ; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH(Increase automatic)
				MOV		P0,#00010000B	; Set Relay ON
				ACALL	DRIVE_LE_CLK

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_WAIT_KEY ; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH(Increase automatic)

				ACALL	WAIT_KEY		; Wait keypad depressed

WAIT_ANYKEY:	JNB		S2_SW,CLR_STATUS; Check S2 pressed?
				JNB		S3_SW,CLR_STATUS; Check S3 pressed?
				MOV		A,P2			; Check Keypad pressed?
				ANL		A,#00FH
				CJNE	A,#00FH,CLR_STATUS
				AJMP	WAIT_ANYKEY		; Nothing key pressed => wait any key
		
CLR_STATUS:		MOV		P0,#00000000B	; Clear Relay Off
				ACALL	DRIVE_LE_CLK

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_EXIT	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH(Increase automatic)
				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s
				ACALL	DELAY_1s
				ACALL	DELAY_1s

				AJMP	LOOP			; Jump to loop

CODE_FAIL:		MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_CODE_FAIL ; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH(Increase automatic)
				MOV		P0,#00000000B	; Clear Relay Off
				ACALL	DRIVE_LE_CLK
				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s
				ACALL	DELAY_1s
				ACALL	DELAY_1s
				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_EXIT	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH(Increase automatic)
				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s
				ACALL	DELAY_1s
				ACALL	DELAY_1s
				AJMP	LOOP			; Jump to loop

;----------------------------------------------------------------------------
; Wait keypad depressed
;----------------------------------------------------------------------------
WAIT_KEY:		MOV		A,P2			; Move P2 to ACC.
				ANL		A,#00FH			; Get only Lower 4 bit
				CJNE	A,#00FH,WAIT_KEY; All bit are high ?
				RET

;----------------------------------------------------------------------------
; Keypad Scan key Subroutine
;----------------------------------------------------------------------------
GET_KPAD:		MOV		P2,#0FFH		; Pull P2 to High (1-Wire not affect)
				MOV		KPAD_DATA,#0	; Clear Keypad Data

CHK_COL0:		CLR		KPAD_COL0		; Begin Scan Column 0
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL0_DETECT ; Check All rows '1'?
				AJMP	CHK_COL1		; All rows '1' => check next column

COL0_DETECT:	MOV		KPAD_DATA,#01	; Initial KPAD_DATA = 1
				AJMP	GET_ROW			; Jump to get row value

CHK_COL1:		SETB	KPAD_COL0		; Stop Scan Column 0
				CLR		KPAD_COL1		; Begin Scan Column 1
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL1_DETECT ; Check All rows '1'?
				AJMP	CHK_COL2		; All rows '1' => check next column

COL1_DETECT:	MOV		KPAD_DATA,#02	; Initial KPAD_DATA = 2
				AJMP	GET_ROW			; Jump to get row value

CHK_COL2:		SETB	KPAD_COL1		; Stop Scan Column 1
				CLR		KPAD_COL2		; Begin Scan Column 2
				MOV		A,P2			; Get Port2 Value
				ANL		A,#00FH			; Get only lower 4 bit
				CJNE	A,#00FH,COL2_DETECT ; Check All rows '1'?
				RET						; All rows '1' => return

COL2_DETECT:	MOV		KPAD_DATA,#03	; Initial KPAD_DATA = 2

GET_ROW:		CLR		KPAD_COL0		; Enable all Column to find Crosspoint
				CLR		KPAD_COL1		;
				CLR		KPAD_COL2		;

				JB		KPAD_ROW0,CHK_ROW1 ; Check Row 0 Detect?
				RET						; Row 0 Detect => return

CHK_ROW1:		JB		KPAD_ROW1,CHK_ROW2 ; Check Row 2 Detect?
				MOV		A,KPAD_DATA		; Add 3 with KPAD_DATA
				ADD		A,#3			;
				MOV		KPAD_DATA,A		;
				RET						; Return

CHK_ROW2:		JB		KPAD_ROW2,CHK_ROW3 ; Check Row 2 Detect?
				MOV		A,KPAD_DATA		; Add 6 with KPAD_DATA
				ADD		A,#6			;
				MOV		KPAD_DATA,A		;
				RET						; Return

CHK_ROW3:		MOV		A,KPAD_DATA		; Add 9 with KPAD_DATA
				ADD		A,#9			;
				MOV		KPAD_DATA,A		;
				RET						; Return

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE		; Driver Latch Enable
				NOP						; NOP
				CLR		DRIVER_LE		; Driver Latch Disable
				NOP
				RET

;----------------------------------------------------------------------------
; Create CG RAM Key Char
;----------------------------------------------------------------------------
LOAD_KEY_CHAR:	MOV		CG_ADDR,#01000000B ; Load CG-RAM Address 00H
				MOV		R0,#0			; Clear loop counter
				MOV		DPTR,#CG_KEY	; Index ROM Pointer
				ACALL	CG_WR_LOOP		; Call CG-RAM Write Routine
				RET

;----------------------------------------------------------------------------
; Create CG RAM Write
;----------------------------------------------------------------------------
CG_WR_LOOP:		CLR		LCD_RS			; Clear LCD_RS
				MOV		P0,CG_ADDR		; Move CG_ADDR to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock

				SETB	LCD_RS			; Set LCD_RS
				CLR		A				; Get ROM Data from Pointer
				MOVC	A,@A+DPTR		;
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock

				INC		DPTR			; Increase Pointer
				INC		CG_ADDR			; Increase CG_ADDR
				INC		R0				; Increase loop counter
				CJNE	R0,#8,CG_WR_LOOP ; Do until 8 times
				RET

;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:		ACALL	DELAY_100ms		; Delay
				CLR		LCD_RS			; Clear LCD_RS Pin

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	DELAY_10ms		; Delay

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_OFF			; Display Off

				ACALL	LCD_CLR			; Clear Display

				MOV		P0,#00000110B	; Entry Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_HOME		; Return Home Display

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000001B	; Display Clear
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000010B	; Return Home
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001000B	; Display Off
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:		SETB	LCD_EN			; Pulse Clock to LCD_EN
				ACALL	LCD_DELAY
				CLR		LCD_EN
				ACALL	LCD_DELAY
				RET

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:			CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001100B	; Display On
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001111B	; Display Cursor and Blink
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:		LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		A,LCD_ADDR		; Move LCD_ADDR to ACC.
				SETB	ACC.7			; Set bit ACC.7
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:		SETB	LCD_RS			; Set LCD_RS Pin
				MOV		P0,LCD_DATA		; Move LCD_DATA to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:		MOV		R0,#0			; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Move data from @DPTR to ACC.
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase loop counter
				CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times

				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 50u, 100u, 1m, 10m, 100m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:		MOV		R7,#002			; Do 2 times
LCD_DELAY_1:	MOV		R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,LCD_DELAY_2
				DJNZ	R7,LCD_DELAY_1
				RET


DELAY_50us:		MOV		R6,#00CH		; Each loop = 50 us
DELAY_50us_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_50us_1
				RET

DELAY_100us:	MOV		R6,#017H		; Each loop = 100 us
DELAY_100us_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100us_1
				RET

DELAY_1ms:		MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_1ms_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_1ms_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;						 0123456789ABCDEF
TITLE_1:		DB		'Combination lock'

TITLE_2:		DB		'Code ',000H,': ','          '

SCR_CODE_PASS:	DB		' Access granted '
SCR_CODE_FAIL:	DB		' Access denied! '
SCR_WAIT_KEY:	DB		' <Exit> Any key '
SCR_EXIT:		DB		'Exit in progress'

CG_KEY:			DB		006H,009H,006H,004H,004H,006H,004H,006H

;						00110	=> 006H
;						01001	=> 009H
;						00110	=> 006H
;						00100	=> 004H
;						00100	=> 004H
;						00110	=> 006H
;						00100	=> 004H
;						00110	=> 006H

CODE_ID:		DB		'*254199#'