;****************************************************************************
; Program		: Stepper Motor 01
; Description	: Stepper Motor 1-Phase Drive
; 				  S2 rotate Left , S3 rotate Right
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1601.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE		BIT		P1.4	; Driver Latch Enable (Active High : Level)

LEFT_SW			BIT		P3.2	; Interrupt 0 PIN S2
RIGHT_SW		BIT		P3.3	; Interrupt 1 PIN S3

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
STEP			EQU		030H	; For keep Stepper State

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				SETB	DRIVER_LE		; Enable Driver Latch

MAIN:			MOV		STEP,#00000001B ; Initial Stepper
				MOV		P0,STEP			; Move Stepper Value to DATABUS

LOOP:			JNB		LEFT_SW,ROTATE_LEFT ; Check S2 Pressed?
				JB		RIGHT_SW,LOOP	; Check S3 Pressed?
				AJMP	ROTATE_RIGHT	; S3 Pressed => Rotate Right

ROTATE_LEFT:	MOV		A,STEP			; Get Stepper State
				SWAP	A				; Swap nibble in ACC.
				CLR		C				; Clear Carry Flag
				RLC		A				; Rotate Left ACC.
				JNC		NEXT_LEFT		; Check Carry Flag
				MOV		A,#00010000B	; Replace new Stepper State Value
NEXT_LEFT:		SWAP	A				; Swap nibble back to original
				MOV		STEP,A			; Save new Stepper State
				MOV		P0,STEP			; Drive new Stepper State
				ACALL	DELAY_100ms		; Delay
				AJMP	LOOP

ROTATE_RIGHT:	MOV		A,STEP			; Get Stepper State
				CLR		C				; Clear Carry Flag
				RRC		A				; Rotate Right ACC.
				JNC		NEXT_RIGHT		; Check Carry Flag
				MOV		A,#00001000B	; Replace new Stepper State Value
NEXT_RIGHT:		MOV		STEP,A			; Save new Stepper State
				MOV		P0,STEP			; Drive new Stepper State
				ACALL	DELAY_100ms		; Delay
				AJMP	LOOP

;----------------------------------------------------------------------------
; Dummy Delay time 100m
;----------------------------------------------------------------------------
DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET