;****************************************************************************
; Program		: R-2R Ladder D/A 01
; Description	: Ramp Signal Generator
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E1701.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
R2R_LE			BIT		P3.5	; R-2R D/A Latch Enable (Active High : Level)

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
R2R_DATA		EQU		030H	; For keep R-2R D/A Data
HEX_VAR			EQU		031H	; For keep HEX Value input variable
BUFFER			EQU		032H	; Define Space 034H-036H for keep DEC. Value

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H		; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		R2R_DATA,#0	; Clear R-2R D/A Data
				MOV		P0,R2R_DATA	; Move R2R Data to DATABUS
				ACALL	R2R_LE_CLK	; Latch

LOOP:			MOV		P0,R2R_DATA	; Move R2R Data to DATABUS
				ACALL	R2R_LE_CLK	; Latch
				INC		R2R_DATA	; Increase R-2R DATA
									; (loop 0 -> FFH -> 0)

				ACALL	DELAY_1ms	; Delay
				AJMP	LOOP		; Jump to loop

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
R2R_LE_CLK:		SETB	R2R_LE		; R-2R Latch Enable
				NOP
				CLR		R2R_LE		; R-2R Latch Disable
				NOP
				RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m
;----------------------------------------------------------------------------
DELAY_1ms:		MOV		R6,#0E6H	; Each loop = 1 ms
DELAY_1ms_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_1ms_1
				RET