;****************************************************************************
; Program		: I2C PCF8591 A/D D/A 01
; Description	: I2C PCF8591 8 bit 4 Ch. A/D , 1 Ch. D/A, Sub address 000
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in PORT1 mode
;
; Filename		: E1901.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
SDA			BIT		P1.0	; SDA I2C Bus
SCL			BIT		P1.1	; SCL I2C Bus

LCD_EN		BIT		P3.6	; LCD Module Enable (Active High : Level)
LCD_RS		BIT		P3.7	; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG		EQU		02FH	; User FLAG
I2C_ACK		BIT		FLAG.0	; Define I2C Acknowledge as bit

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR		EQU		030H		; For keep LCD Address
LCD_DATA		EQU		031H		; For keep LCD Data
LCD_PTR			EQU		032H		; For keep LCD 3 Cha.r Pointer
I2C_ADDR		EQU		033H		; For keep I2C Address
I2C_DATA		EQU		034H		; For keep I2C Data
CONTROL			EQU		035H		; For keep I2C PCF8591 Control Byte
DA_DATA			EQU		036H		; For keep I2C D/A Data
CHANNEL			EQU		037H		; For keep Channel Number
AD_DATA			EQU		038H		; For keep I2C A/D Data
BUFFER			EQU		039H		; Buffer
									; 39H : Ch.0
									; 3AH : Ch.1
									; 3BH : Ch.2
									; 3CH : Ch.3

;----------------------------------------------------------------------------
; Define I2C Slave Address
;----------------------------------------------------------------------------
PCF8591_ID		EQU		10010000B ; PCF8591:000 Slave Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			ACALL	INIT_LCD		; Call LCD Initial subroutine

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-0FH(Increase automatic)

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#TITLE_2	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 40H-4FH(Increase automatic)

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s		; Delay

				MOV		DA_DATA,#0		; Clear D/A Data

MAIN_LOOP:		MOV		CHANNEL,#0		; Initial Channel 0
				MOV		R1,#BUFFER		; Set Start Buffer Pointer
				MOV		R4,#4			; Set loop 4 times

CONVERSION_LOOP:	MOV		A,CHANNEL	; Get Channel
				ADD		A,#01000000B	; Add Channel to Control Byte
				MOV		CONTROL,A		; Set Control 4 input , Analog Out En.
				ACALL	PCF8591_WR		; Write PCF8591

				ACALL	PCF8591_RD		; Read PCF8591		; Repeat Read to
				ACALL	PCF8591_RD		; Read PCF8591		; Get Previos Data
				MOV		@R1,AD_DATA		; Read A/D Data to @R1
				INC		R1				; Increase Buffer Pointer
				INC		CHANNEL			; Increase Channel
				DJNZ	R4,CONVERSION_LOOP ; Do until 4 times

				ACALL	LCD_CLR

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_DA	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

				MOV		LCD_ADDR,#00CH	; Set Address 0CH
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,DA_DATA ; Write D/A Data Out
				ACALL	HEX2LCD			; Show Data as HEX Code

				INC		DA_DATA			; Increase D/A Value

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s		; Delay

				MOV		R1,#BUFFER		; Set Start Buffer Pointer

				MOV		LCD_ADDR,#000H	; Set Address 00H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_AD01	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

				MOV		LCD_ADDR,#004H	; Set Address 04H
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
				ACALL	HEX2LCD			; Show Data as HEX Code

				INC		R1				; Increase Buffer Pointer

				MOV		LCD_ADDR,#00CH	; Set Address 0CH
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
				ACALL	HEX2LCD			; Show Data as HEX Code

				INC		R1				; Increase Buffer Pointer

				MOV		LCD_ADDR,#040H	; Set Address 40H
				ACALL	SET_ADDR_LCD	;
				MOV		DPTR,#SCR_AD23	; Index Pointer ROM to Show LCD
				ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)

				MOV		LCD_ADDR,#044H	; Set Address 44H
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
				ACALL	HEX2LCD			; Show Data as HEX Code

				INC		R1				; Increase Buffer Pointer

				MOV		LCD_ADDR,#04CH	; Set Address 4CH
				ACALL	SET_ADDR_LCD	;
				MOV		LCD_DATA,@R1	; Move Data Buffer with Pointer to LCD
				ACALL	HEX2LCD			; Show Data as HEX Code

				ACALL	DELAY_1s		; Delay
				ACALL	DELAY_1s		; Delay

				AJMP	MAIN_LOOP		; Jump to loop

;----------------------------------------------------------------------------
; HEX Code to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
HEX2LCD:		PUSH	ACC				; Push ACC.
				MOV		A,LCD_DATA		; Get Data
				MOV		B,#16			;
				DIV		AB				; Divide by 16
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		;
				ACALL	HEX_CHK			; Check A-F
				ACALL	WRCHAR_LCD		; Write Higher HEX Code
				MOV		A,B				; Get Remainder
				ADD		A,#030H			; Convert to ASCII
				MOV		LCD_DATA,A		;
				ACALL	HEX_CHK			; Check A-F
				ACALL	WRCHAR_LCD		; Write Lower HEX Code
				POP		ACC				; Pop ACC.
				RET						; Return

HEX_CHK:		MOV		A,LCD_DATA		; Get Data
				CJNE	A,#03AH,CHK_OTHER ; Check Data Equal or Greater than A
CHK_OTHER:		JNC		CONV_2_ALPHA	; If Equal or Greater than => A-F
				RET						; Return

CONV_2_ALPHA:	ADD		A,#7			; Add 7 to Alphabel
				MOV		LCD_DATA,A		; Save Data
				RET						; Return

;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:		ACALL	DELAY_100ms		; Delay
				CLR		LCD_RS			; Clear LCD_RS Pin

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	DELAY_10ms		; Delay

				MOV		P0,#00111000B	; 8bit Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_OFF			; Display Off

				ACALL	LCD_CLR			; Clear Display

				MOV		P0,#00000110B	; Entry Mode
				ACALL	LCD_CLK			; Pulse LCD Clock

				ACALL	LCD_HOME		; Return Home Display

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000001B	; Display Clear
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00000010B	; Return Home
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:		CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001000B	; Display Off
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:		SETB	LCD_EN			; Pulse Clock to LCD_EN
				ACALL	LCD_DELAY
				CLR		LCD_EN
				ACALL	LCD_DELAY
				RET

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:			CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		P0,#00001100B	; Display On
				ACALL	LCD_CLK
				RET

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:		LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR		LCD_RS			; Clear LCD_RS Pin
				MOV		A,LCD_ADDR		; Move LCD_ADDR to ACC.
				SETB	ACC.7			; Set bit ACC.7
				MOV		P0,A			; Move to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				RET

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:		LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:		SETB	LCD_RS			; Set LCD_RS Pin
				MOV		P0,LCD_DATA		; Move LCD_DATA to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:		MOV		R0,#0			; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS			; Set LCD_RS Pin
				CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Move data from @DPTR to ACC.
				MOV		P0,A			; Move ACC. to DATABUS
				ACALL	LCD_CLK			; Pulse LCD Clock
				INC		DPTR			; Increase Pointer
				INC		R0				; Increase loop counter
				CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
				ACALL	LCD_ON			; Display On
				RET

;----------------------------------------------------------------------------
; I2C PCF8591 Read
;----------------------------------------------------------------------------
PCF8591_RD:		MOV		I2C_ADDR,#PCF8591_ID+1 ; Set PCF8591 as I2C Read Slave
				ACALL	I2C_SLAVE		; Connect Slave

				ACALL	I2C_DATA_RD		; Read Data from Slave
				MOV		AD_DATA,I2C_DATA ; Read Data to A/D Data
				ACALL	I2C_NACK_BIT	; Send Not Acknowledge

				ACALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C PCF8591 Write
;----------------------------------------------------------------------------
PCF8591_WR:		MOV		I2C_ADDR,#PCF8591_ID ; Set PCF8591 as I2C Write Slave
				ACALL	I2C_SLAVE		; Connect Slave

				MOV		I2C_DATA,CONTROL ; Write CONTROL to Slave
				ACALL	I2C_DATA_WR		; Write Data to Slave

				MOV		I2C_DATA,DA_DATA ; Write D/A Data to Slave
				ACALL	I2C_DATA_WR		; Write Data to Slave

				ACALL	I2C_STOP		; Send Stop Condition
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Write
; I/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_WR:	PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_DATA		; Get Data
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_WR_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_DATA_WR_1	; Do until 8 times
				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_DATA_WR_2	; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK. bit
I2C_DATA_WR_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Read
; O/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_RD:	PUSH	ACC				; Push ACC.
				CLR		A				; Clear ACC.
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_RD_1:	ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				MOV		C,SDA			; Get SDA to Carry Flag
				RLC		A				; Rotate ACC. to Left with Carry
				CLR		SCL				; Clear SCL
				DJNZ	R5,I2C_DATA_RD_1; Do until 8 times
				MOV		I2C_DATA,A		; Move Data to I2C_DATA
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Slave Connect
; I/P:		I2C_ADDR
; O/P Flag:	I2C_ACK
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_SLAVE:		PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_ADDR		; Get Slave Address
				ACALL	I2C_START		; Send Start Condition
				MOV		R5,#008			; Set loop 8 times
I2C_SLAVE_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_SLAVE_1	; Do until 8 times

				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_SLAVE_2 ; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK.
I2C_SLAVE_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Start Condition
;----------------------------------------------------------------------------
I2C_START:		JNB		SCL,I2C_START_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_START_1:	SETB	SDA				; Set SDA
				SETB	SCL				; Set SCL

				ACALL	I2C_DELAY		; Delay
				CLR		SDA				; Clear SDA during SCL set
				ACALL	I2C_DELAY		; Delay
				CLR		SCL				; Clear SCL
				RET						; Return

;----------------------------------------------------------------------------
; I2C Stop Condition
;----------------------------------------------------------------------------
I2C_STOP:		JNB		SCL,I2C_STOP_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_STOP_1:		CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				SETB	SDA				; Set SDA during SCL set
				RET						; Return

;----------------------------------------------------------------------------
; I2C Clock
;----------------------------------------------------------------------------
I2C_CLK:		ACALL	I2C_DELAY		; Pulse SCL
				SETB	SCL				;
				ACALL	I2C_DELAY		;
				CLR		SCL				;
				RET						; Return

;----------------------------------------------------------------------------
; I2C Acknowledge
;----------------------------------------------------------------------------
I2C_ACK_BIT:	CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SDA
				RET						; Return

;----------------------------------------------------------------------------
; I2C Not Acknowledge
;----------------------------------------------------------------------------
I2C_NACK_BIT:	SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SCL
				RET						; Return

;----------------------------------------------------------------------------
; Dummy Delay time I2C_DELAY, LCD_DELAY, 10ms, 100m, 1s
;----------------------------------------------------------------------------
I2C_DELAY:		MOV		R6,#00CH		; Each loop = 50 us
I2C_DELAY_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,I2C_DELAY_1
				RET

LCD_DELAY:		MOV		R7,#002			; Do 2 times
LCD_DELAY_1:	MOV		R6,#0E6H		; Each loop = 1 ms
LCD_DELAY_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,LCD_DELAY_2
				DJNZ	R7,LCD_DELAY_1
				RET

DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;						 0123456789ABCDEF
TITLE_1:		DB		'  I2C  PCF8591  '
TITLE_2:		DB		' A/D, D/A  8bit '
SCR_AD01:		DB		'Ch1:  H Ch2:  H '
SCR_AD23:		DB		'Ch3:  H Ch4:  H '
SCR_DA:			DB		'Analog Out :   H'