;****************************************************************************
; Program		: Serial UART 02
; Description	: Serial UART Exchange Data with PC
;				: using Bit polled check method
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E2502.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
BUFFER			EQU		030H	; Buffer

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector

										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

MAIN:			MOV		TMOD,#021H		; T1 8Bit Auto, T0 16Bit
				MOV		TH1,#0FAH		; 9600 bps Timer1 Default
				MOV		TL1,#0FAH		;

				SETB	TR1				; Start Timer1

				MOV		SCON,#040H		; Mode1 RX Disable

				MOV		DPTR,#SERIAL_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		DPTR,#HELLO_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

START_LOOP:		MOV		R0,#BUFFER		; Clear BUFFER to start pointer
				SETB	REN				; Set RX Enable

LOOP:			JNB		RI,$			; Wait byte received
				CLR		RI				;
				MOV		A,SBUF			; Get byte from SBUF
				MOV		@R0,A			;
				CJNE	A,#00DH,RX_NEXT ; Check Enter key receive?
				
				MOV		DPTR,#GOT_TEXT1	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				
				MOV		A,R0			; Keep end of message pointer

				MOV		R0,#BUFFER		; Reset Pointer
REPLY_LOOP:		CLR		TI				; Send reply byte to PC
				MOV		SBUF,@R0		; by get data from BUFFER
				JNB		TI,$			;
				CLR		TI				;

				INC		R0				; Increase to next pointer
				CJNE	A,0,REPLY_LOOP	; Check end of message already?

				MOV		DPTR,#GOT_TEXT2	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				AJMP	START_LOOP
				
RX_NEXT:		INC		R0              ; Increase pointer
				AJMP	LOOP			; Back to do loop

;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:		CLR		TI				; Clear TI
TX_LOOP:		CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				INC		DPTR			; Increase Pointer
				CJNE	A,#0FFH,TX_CHAR ; Check 0FFH End of Text Char.
				RET						; End => Return

TX_CHAR:		MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				AJMP	TX_LOOP			; Jump to TX_LOOP

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
SERIAL_TEXT:	DB		00AH,00DH
				DB		00AH,00DH
				DB		'Serial UART Communication < baudrate 9600bps > ',00AH,00DH
				DB		'for MCS-51 Microcontroller Training System',00AH,00DH
				DB		'Copyright (C) 2000 Innovative Experiment Co.,Ltd.',00AH,00DH
				DB		00AH,00DH,0FFH
				
HELLO_TEXT:		DB		'Hello.. PC. I',027H,'m waiting message from you.',00AH,00DH
				DB		'type any message then press ENTER key.',00AH,00DH,0FFH                                

GOT_TEXT1:		DB		'I',027H,'d got message [',0FFH
GOT_TEXT2:		DB		'] from you.',00AH,00DH,0FFH