;****************************************************************************
; Program		: Serial UART 03
; Description	: Serial UART Exchange Data with PC
;				: using Serial interrupt
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: P2503.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

RX_OK			BIT		F0		; Define bit to keep received already

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
TX_DATA			EQU		02FH	; For keep TX Data
BUFFER			EQU		030H	; Buffer

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector
				AJMP	MAIN			; Jump to Main

				ORG		0023H			; TI+RI Vector
				AJMP	SER_INT			; Goto Serial interrupt subroutine

MAIN:									; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				MOV		TMOD,#021H		; T1 8Bit Auto, T0 16Bit
				MOV		TH1,#0FAH		; 9600 bps Timer1 Default
				MOV		TL1,#0FAH		;

				MOV		IE,#10010000B	; En. EA, ES
				SETB	TR1				; Start Timer1

				MOV		SCON,#050H		; Mode1 RX Enable
				CLR		RX_OK			; Clear RX_OK flag

				MOV		DPTR,#SERIAL_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		DPTR,#HELLO_TEXT ; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

START_LOOP:		MOV		R0,#BUFFER		; Clear BUFFER to start pointer

LOOP:			JNB		RX_OK,$			; Wait message received

				MOV		DPTR,#GOT_TEXT1	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				ACALL	TX_BUFFER		; TX Buffer to Serial Port

				MOV		DPTR,#GOT_TEXT2	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				AJMP	START_LOOP		; Back to received next message
				
;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:		CLR		TI				; Clear TI
TX_TEXT_LOOP:	CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				INC		DPTR			; Increase Pointer
				CJNE	A,#0FFH,TX_TEXT_CHAR ; Check 0FFH End of Text Char.
				RET						; End => Return

TX_TEXT_CHAR:	MOV		TX_DATA,A		; Send Data to SBUF
				SETB	TI				; Set TI to SER_INT subroutine
				AJMP	TX_TEXT_LOOP	; Wait for TX

;----------------------------------------------------------------------------
; TX Buffer Text from ROM Pointer
;----------------------------------------------------------------------------
TX_BUFFER:		CLR		TI				; Clear TI
				MOV		R0,#BUFFER		; Set BUFFER to start pointer
TX_BUFFER_LOOP:	MOV		A,@R0			; Get Data from ROM with buffer offset
				INC		R0				; Increase buffer pointer
				CJNE	A,#00DH,TX_BUFFER_CHAR ; Check 0FFH End of Text Char.
				CLR		RX_OK			; Clear RX_OK flag after TX all
				RET						; End => Return

TX_BUFFER_CHAR:	MOV		TX_DATA,A		; Send Data to SBUF
				SETB	TI				; Set TI to SER_INT subroutine
				AJMP	TX_BUFFER_LOOP	; Wait for TX

;----------------------------------------------------------------------------
; Serial interrupt subroutine
;----------------------------------------------------------------------------
SER_INT:		PUSH	ACC				; Push ACC. to stack

				JBC		RI,SER_RX		; RX or TX? If RX then received and clear RI
SER_TX:			CLR		TI				; Clear TI
				MOV		SBUF,TX_DATA
SER_TX_WAIT:	JBC		TI,SER_EXIT		; Wait until TX already (TI=1) then clear TI
				AJMP	SER_TX_WAIT		; Wait for TX

SER_RX:			CLR		RX_OK			; Clear RI
				MOV		A,SBUF			; Get data to ACC.
				MOV		@R0,A			; Move to Buffer @R0
				INC		R0				; Increase R0
				CJNE	A,#00DH,SER_EXIT	; Check 00DH End of Buffer text
				SETB	RX_OK			; Set RX_OK received message ok.

SER_EXIT:		POP		ACC				; Pop Acc. to Main
				RETI					; Return interrupt

;----------------------------------------------------------------------------
; Dummy Delay time 100m
;----------------------------------------------------------------------------
DELAY_100ms:	MOV		R7,#100			; Do 100 times
DELAY_100ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_100ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_100ms_2
				DJNZ	R7,DELAY_100ms_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
SERIAL_TEXT:	DB		00AH,00DH
				DB		00AH,00DH
				DB		'Serial UART Communication < baudrate 9600bps > ',00AH,00DH
				DB		'for MCS-51 Microcontroller Training System',00AH,00DH
				DB		'Copyright (C) 2000 Innovative Experiment Co.,Ltd.',00AH,00DH
				DB		00AH,00DH,0FFH
				
HELLO_TEXT:		DB		'Hello.. PC. I',027H,'m waiting message from you.',00AH,00DH
				DB		'type any message then press ENTER key.',00AH,00DH,0FFH                                

GOT_TEXT1:		DB		'I',027H,'d got message [',0FFH
GOT_TEXT2:		DB		'] from you.',00AH,00DH,0FFH