;****************************************************************************
; Program		: Serial EEPROM 24C16
; Description	: Read/Write Serial EEPROM 24C16
;				: using Terminal on PC
;				; Serial UART Bit polled check method
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in PORT1 mode
;
; Filename		: E2601.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
SDA				BIT		P1.0	; SDA I2C Bus
SCL				BIT		P1.1	; SCL I2C Bus

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02AH	; User FLAG
I2C_ACK			BIT		FLAG.0	; Define I2C Acknowledge as bit
RX_ERR			BIT		FLAG.1	; Define RX error as bit

EEP_DATA		EQU		02BH    ; For keep AT24C16 data
EEP_AL			EQU		02CH    ; For keep AT24C16 lower address
EEP_AH			EQU		02DH	; For keep AT24C16 higher address

I2C_ADDR		EQU		02EH	; For keep I2C Address
I2C_DATA		EQU		02FH	; For keep I2C Data

BUFFER			EQU		030H	; Buffer

;----------------------------------------------------------------------------
; Define I2C Slave Address
;----------------------------------------------------------------------------
AT24C16_ID      EQU     10100000B ; AT24C16 Slave Address

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------
                                                      
				ORG		0000H			; Reset Vector

										; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				MOV		TMOD,#021H		; T1 8Bit Auto, T0 16Bit
				MOV		TH1,#0FAH		; 9600 bps Timer1 Default
				MOV		TL1,#0FAH		;

				SETB	TR1				; Start Timer1

				MOV		SCON,#050H		; Mode1 RX Enabled

MAIN:			MOV		DPTR,#MENU		; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				ACALL	RX_BYTE			; Get byte from PC
				CJNE	A,#'W',MENU_R	; Check Enter key receive?
MENU_WRITE:		MOV		DPTR,#WR_ADDR	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		R0,#BUFFER		; Set start buffer
				MOV		R4,#3			; Get 3 key as data
GET_ADDR_LOOP:	ACALL	RX_BYTE			;
				ACALL	CHECK_HEX		; Check data valid
				JB		RX_ERR,MENU_WRITE ; If not valid then re-enter
				MOV		@R0,A			; Kepp data to buffer
				INC		R0				; Increase pointer
GET_ADDR_NEXT:	DJNZ	R4,GET_ADDR_LOOP ; Do until 3 times

				MOV		A,#00AH			; Send line feed to PC
				ACALL	TX_BYTE			;

DATA_WRITE:		MOV		DPTR,#WR_DATA	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		R0,#BUFFER+3	; Set buffer to block 3
				MOV		R4,#2			; Get 2 key as data
GET_DATA_LOOP:	ACALL	RX_BYTE			;
				ACALL	CHECK_HEX		; Check data valid
				JB		RX_ERR,DATA_WRITE ; If not valid then re-enter
				MOV		@R0,A			; Keep data to buffer
				INC		R0				; Increase pointer
GET_DATA_NEXT:	DJNZ	R4,GET_DATA_LOOP ; Do until 2 times

				ACALL	BUFFER_CONV		; Convert 5 byte address as EEP_AH,EEP_AL and EEP_DATA
				ACALL	AT24C16_WR		; Write data to AT24C16

                JB		I2C_ACK,ERROR	; If not acknowledge, show error

				MOV		DPTR,#WRITE_OK	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				AJMP	MAIN			; Back to main

MENU_R:			CJNE	A,#'R',MAIN	; Check Enter key receive?
MENU_READ:		MOV		DPTR,#READ_TEXT	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		R0,#BUFFER		; Set start buffer
				MOV		R4,#3			; Get 3 key as address
GET_RD_ADDR_LOOP:	ACALL	RX_BYTE		; 
				ACALL	CHECK_HEX		; Check data valid
				JB		RX_ERR,MENU_READ ; If not valid then re-enter
				MOV		@R0,A			; Keep data to buffer
				INC		R0				; Increase pointer
GET_RD_ADDR_NEXT:	DJNZ	R4,GET_RD_ADDR_LOOP ; Do until 3 times

				MOV		DPTR,#READ_AS	; Set Pointer Serial TX
				ACALL	TX_TEXT         ; TX Text to Serial Port

				ACALL	BUFFER_CONV		; Convert 3 byte address as EEP_AH and EEP_AL
										; , but ignore EEP_DATA
				ACALL	AT24C16_RD		; Read data from AT24C16

				MOV		A,#' '			; Send space
				ACALL	TX_BYTE			;
				
                JB		I2C_ACK,ERROR	; If not acknowledge, show error

				MOV		A,EEP_DATA		; Send data to terminal
				MOV		B,A				;
				ANL		A,#0F0H			;
				SWAP	A				;
				ACALL	TX_ASCII		; Send higher 4 bit as ASCII

				MOV		A,B             ;
				ANL		A,#00FH         ;
				ACALL	TX_ASCII        ; Send lower 4 bit as ASCII

				AJMP	MAIN			; Back to main

ERROR:			MOV		DPTR,#ACK_ERR	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				AJMP	MAIN			; Goto main

;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:		CLR		TI				; Clear TI
TX_LOOP:		CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				INC		DPTR			; Increase Pointer
				CJNE	A,#0FFH,TX_CHAR ; Check 0FFH End of Text Char.
				RET						; End => Return

TX_CHAR:		MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				AJMP	TX_LOOP			; Jump to TX_LOOP

;----------------------------------------------------------------------------
; TX byte to serial
;----------------------------------------------------------------------------
TX_BYTE:		MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				RET						; Return

;----------------------------------------------------------------------------
; TX ASCII to serial
;----------------------------------------------------------------------------
TX_ASCII:		ADD		A,#030H			; Add 030H to convert to number
				CJNE	A,#03AH,TX_ASCII_1;
TX_ASCII_1:		JC		TX_ASCII_NEXT   ; If data in 03AH-03FH add 007H to convert to alphabet
				ADD		A,#007H			;
TX_ASCII_NEXT:	MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				RET						; Return

;----------------------------------------------------------------------------
; RX byte from serial
;----------------------------------------------------------------------------
RX_BYTE:		JNB		RI,$			; Wait byte received
				CLR		RI				;
				MOV		A,SBUF			; Get byte from SBUF
				ACALL	UPCASE_ACC		; Upcase ACC
				RET						; Return

;----------------------------------------------------------------------------
; Upcase ACC.
;----------------------------------------------------------------------------
UPCASE_ACC:		CJNE	A,#07BH,UPCASE_ACC_1	; Check data valid in small case?
UPCASE_ACC_1:	JNC		UPCASE_EXIT				; If data => 07BH then exit
				CJNE	A,#061H,UPCASE_ACC_2	;
UPCASE_ACC_2:	JC		UPCASE_EXIT             ; or data < 061H then exit
				SUBB	A,#020H					; Else ubtract with 020H to UPCASE data
UPCASE_EXIT:	RET								; Return

;----------------------------------------------------------------------------
; Check data valid on hexadecimal convert ASCII to HEX & Reply when OK
;----------------------------------------------------------------------------
CHECK_HEX:		SETB	RX_ERR					; Set RX_ERR bit
				CJNE	A,#'0',CHECK_HEX_1		; Check data valid in number?
CHECK_HEX_1:	JC 		CHECK_HEX_EXIT			; If data < 030H then ERROR
				CJNE	A,#'9'+1,CHECK_HEX_2	;
CHECK_HEX_2:	JC		CHECK_HEX_OK			; Data valid on 030H-039H then OK
				CJNE	A,#'A',CHECK_HEX_3	;
CHECK_HEX_3:	JC		CHECK_HEX_EXIT			; Data valid on 03AH-040H then ERROR
				CJNE	A,#'F'+1,CHECK_HEX_4	;
CHECK_HEX_4:	JNC		CHECK_HEX_EXIT			; If data valid on 'A'-'F' then OK, else ERROR
				ACALL	TX_BYTE					; Reply
				CLR		C						; Subtract with 037H to convert to 00AH-00FH
				SUBB	A,#037H					;
				AJMP	CHECK_HEX_NEXT          ; Jump to next

CHECK_HEX_OK:	ACALL	TX_BYTE					; Reply
				CLR		C						; Subtract with 030H to convert to 000H-009H
				SUBB	A,#030H					;
CHECK_HEX_NEXT:	CLR		RX_ERR					; Clear RX_ERR when data valid
CHECK_HEX_EXIT:	RET								; Return

;----------------------------------------------------------------------------
; Buffer convert to EEP_AH,EEP_AL,EEP_DATA
;----------------------------------------------------------------------------
BUFFER_CONV:	MOV		R0,#BUFFER				; Get buffer pointer
				MOV		A,@R0					; Get first data as EEP_AH
				MOV		EEP_AH,A				;
				INC		R0						; Get next buffer
				MOV		A,@R0                   ; as higher 4 bit of EEP_AL
				SWAP	A						;
				INC		R0						; Get next buffer
				MOV		EEP_AL,@R0				; as lower 4 bit of EEP_AL
				ADD		A,EEP_AL                ; add higher 4 bit to complete EEP_AL
				MOV		EEP_AL,A				;

				INC		R0						; Get next buffer
				MOV		A,@R0					; as higher 4 bit of EEP_DATA
				SWAP	A						;
				INC		R0						; Get next buffer
				MOV		EEP_DATA,@R0			; as lower 4 bit of EEP_DATA
				ADD		A,EEP_DATA				; add higher 4 bit to compleet EEP_AH
				MOV		EEP_DATA,A				;
				
				RET								; Return

;----------------------------------------------------------------------------
; I2C AT24C16 Read
;----------------------------------------------------------------------------
AT24C16_RD:     MOV     A,EEP_AH		; Move ACC with 3 bit higher address
				RL		A				; Shift to left 1 times
				ADD		A,#AT24C16_ID 	; Add with AT24C16 ID as I2C Write Slave
				MOV		I2C_ADDR,A		;
                ACALL   I2C_SLAVE       ; Connect Slave
				JB		I2C_ACK,AT24C16_RD_EXIT

                MOV     I2C_DATA,EEP_AL ; Write low address to Slave
                ACALL   I2C_DATA_WR     ; Write Data to Slave

				MOV     A,EEP_AH		; Move ACC with 3 bit higher address
				RL		A				; Shift to left 1 times
				ADD     A,#AT24C16_ID+1 ; Set AT24C16 as I2C Read Slave
				MOV		I2C_ADDR,A		;
                ACALL   I2C_SLAVE       ; Connect Slave

                ACALL   I2C_DATA_RD     ; Read Data from Slave
                MOV     EEP_DATA,I2C_DATA ; Read Data to EEP_DATA
                ACALL   I2C_NACK_BIT    ; Send Not Acknowledge

                ACALL   I2C_STOP        ; Send Stop Condition
AT24C16_RD_EXIT: RET                     ; Return

;----------------------------------------------------------------------------
; I2C AT24C16 Write
;----------------------------------------------------------------------------
AT24C16_WR:     MOV     A,EEP_AH		; Move ACC with 3 bit higher address
				RL		A				; Shift to left 1 times
				ADD     A,#AT24C16_ID ; Set AT24C16 as I2C Write Slave
				MOV		I2C_ADDR,A		;
                ACALL   I2C_SLAVE       ; Connect Slave
				JB		I2C_ACK,AT24C16_WR_EXIT

                MOV     I2C_DATA,EEP_AL ; Write Pointer to Slave
                ACALL   I2C_DATA_WR     ; Write Data to Slave

                MOV     I2C_DATA,EEP_DATA ; Write EEP_DATA to Slave
                ACALL   I2C_DATA_WR     ; Write Data to Slave

                ACALL   I2C_STOP        ; Send Stop Condition
AT24C16_WR_EXIT: RET                     ; Return

;----------------------------------------------------------------------------
; I2C Data Write
; I/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_WR:	PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_DATA		; Get Data
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_WR_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_DATA_WR_1	; Do until 8 times
				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_DATA_WR_2	; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK. bit
I2C_DATA_WR_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Data Read
; O/P:		I2C_DATA
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_DATA_RD:	PUSH	ACC				; Push ACC.
				CLR		A				; Clear ACC.
				MOV		R5,#008			; Set loop 8 times
I2C_DATA_RD_1:	ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				MOV		C,SDA			; Get SDA to Carry Flag
				RLC		A				; Rotate ACC. to Left with Carry
				CLR		SCL				; Clear SCL
				DJNZ	R5,I2C_DATA_RD_1; Do until 8 times
				MOV		I2C_DATA,A		; Move Data to I2C_DATA
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Slave Connect
; I/P:		I2C_ADDR
; O/P Flag:	I2C_ACK
; Reserve:	R5
;----------------------------------------------------------------------------
I2C_SLAVE:		PUSH	ACC				; Push ACC.
				SETB	I2C_ACK			; Set ACK. bit
				MOV		A,I2C_ADDR		; Get Slave Address
				ACALL	I2C_START		; Send Start Condition
				MOV		R5,#008			; Set loop 8 times
I2C_SLAVE_1:	RLC		A				; Rotate ACC. to Left with Carry
				MOV		SDA,C			; Move Carry Flag to SDA
				ACALL	I2C_CLK			; Pulse I2C Clock
				DJNZ	R5,I2C_SLAVE_1	; Do until 8 times

				SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				JB		SDA,I2C_SLAVE_2 ; Check Acknowledge from Slave
				CLR		I2C_ACK			; Clear ACK.
I2C_SLAVE_2:	CLR		SCL				; Clear SCL
				POP		ACC				; Pop ACC.
				RET						; Return

;----------------------------------------------------------------------------
; I2C Start Condition
;----------------------------------------------------------------------------
I2C_START:		JNB		SCL,I2C_START_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_START_1:	SETB	SDA				; Set SDA
				SETB	SCL				; Set SCL

				ACALL	I2C_DELAY		; Delay
				CLR		SDA				; Clear SDA during SCL set
				ACALL	I2C_DELAY		; Delay
				CLR		SCL				; Clear SCL
				RET						; Return

;----------------------------------------------------------------------------
; I2C Stop Condition
;----------------------------------------------------------------------------
I2C_STOP:		JNB		SCL,I2C_STOP_1	; Check current SCL set?
				CLR		SCL				; Clear SCL

I2C_STOP_1:		CLR		SDA				; Clear SDA
				ACALL	I2C_DELAY		; Delay
				SETB	SCL				; Set SCL
				ACALL	I2C_DELAY		; Delay
				SETB	SDA				; Set SDA during SCL set
				RET						; Return

;----------------------------------------------------------------------------
; I2C Clock
;----------------------------------------------------------------------------
I2C_CLK:		ACALL	I2C_DELAY		; Pulse SCL
				SETB	SCL				;
				ACALL	I2C_DELAY		;
				CLR		SCL				;
				RET						; Return

;----------------------------------------------------------------------------
; I2C Not Acknowledge
;----------------------------------------------------------------------------
I2C_NACK_BIT:	SETB	SDA				; Set SDA
				ACALL	I2C_DELAY		; Delay
				ACALL	I2C_CLK			; Pulse I2C Clock
				SETB	SCL
				RET						; Return

;----------------------------------------------------------------------------
; Dummy Delay time I2C_DELAY
;----------------------------------------------------------------------------
I2C_DELAY:		MOV		R6,#00CH		; Each loop = 50 us
I2C_DELAY_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,I2C_DELAY_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
MENU:			DB		00AH,00DH
				DB		00AH,00DH
				DB		00AH,00DH
				DB		'Serial EEPROM AT24C16',00AH,00DH
				DB		'Press "R" to read or "W" to write data',0FFH

WR_ADDR:		DB		00AH,00DH
				DB		'Write to address (000H-7FFH) : ',0FFH

WR_DATA:		DB		00DH,'with data (00H-FFH) : ',0FFH

READ_TEXT:		DB		00AH,00DH
				DB		'Read data at (000H-7FFH) : ',0FFH

READ_AS:		DB		' = ',0FFH
				
WRITE_OK:		DB		' Write OK.',00AH,00DH,0FFH

ACK_ERR:		DB		' Error! AT24C16 not acknowledge ',00AH,00DH,0FFH