;****************************************************************************
; Program		: Pulse width modulation by PCA
; Description	: Generate pulse width modulation signal on CEX2,CEX3 and CEX4
;				: using PCA function
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
;
; NX-51 V2 NOTE : P1 Function switch must be in T2/PCA mode
;
; Filename		: E2801.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define MCS-51 PCA Register
; (This section may not define on other assembler)
;----------------------------------------------------------------------------
CCON			EQU		0D8H	; CCON Adress in SFR
CMOD			EQU		0D8H	; CCON Adress in SFR
CL				EQU		0E9H	; CL Adress in SFR
CH				EQU		0F9H	; CH Adress in SFR

CCAPM2			EQU		0DCH	; CCAPM2 Adress in SFR
CCAPM3			EQU		0DDH	; CCAPM3 Adress in SFR
CCAPM4			EQU		0DEH	; CCAPM4 Adress in SFR

CCAP2L			EQU		0ECH	; CCAP2L Adress in SFR
CCAP3L			EQU		0EDH	; CCAP3L Adress in SFR
CCAP4L			EQU		0EEH	; CCAP4L Adress in SFR

CCAP2H			EQU		0FCH	; CCAP2H Adress in SFR
CCAP3H			EQU		0FDH	; CCAP3H Adress in SFR
CCAP4H			EQU		0FEH	; CCAP4H Adress in SFR

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector

MAIN:									; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				MOV		CMOD,#000H		; Set CMOD PCA count pulse as  (osc/12)

				MOV		CCAPM2,#042H	; Set CCAP Module 2 as 8bit PWM
				MOV		CCAPM3,#042H	; Set CCAP Module 3 as 8bit PWM
				MOV		CCAPM4,#042H	; Set CCAP Module 4 as 8bit PWM

				MOV		CCAP2L,#51		; Set CCAP2L reload at 20% duty cycle
				MOV		CCAP2H,#51		; Set CCAP2H reload at 20% duty cycle

				MOV		CCAP3L,#128		; Set CCAP3L reload at 50% duty cycle
				MOV		CCAP3H,#128		; Set CCAP3H reload at 50% duty cycle

				MOV		CCAP4L,#204 	; Set CCAP4L reload at 80% duty cycle
				MOV		CCAP4H,#204 	; Set CCAP4H reload at 80% duty cycle

				MOV		CCON,#040H		; Set PCA counter run

				AJMP	$