;****************************************************************************
; Program		: Room temperature data logger
; Description	: Room temperature data logger use DS1820
;				: 60 records, sampling 1 second, using internal RAM to keep data
; For			: NX-51 Version 2.0 (Philips P89C51RD2)
;				: MCS-51 Microcontroller Training System
; Filename		: E2901.ASM
; Assembler		: RAD51
; Copyright (C) 2000 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
ONEWIRE			BIT		P2.7		; 1-Wire Interface (DS1820 Temp. Sensor)

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
FLAG			EQU		02FH		; User FLAG
BUSY			BIT		FLAG.0		; Define BUSY as bit

ONEWIRE_DATA	EQU		032H		; For keep ONEWIRE Data
TEMP			EQU		033H		; For keep Temp. Data (Temp L only)

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

				ORG		0000H			; Reset Vector

MAIN:									; Initial all port for NX-51 V.2.0
				MOV		P0,#00000000B	; Clear Databus
				MOV		P1,#11101111B	; Clear status all devices
				MOV		P2,#11111111B   ; Free Keypad input and onewire
				MOV		P3,#00011111B	; Clear status LCD, R-2R, P3.2-3.4

				MOV		TMOD,#021H		; T1 8Bit Auto, T0 16Bit
				MOV		TH1,#0FAH		; 9600 bps Timer1 Default
				MOV		TL1,#0FAH		;

				SETB	TR1				; Start Timer1

				MOV		SCON,#050H		; Mode1 RX enable

MENU_MAIN:		MOV		DPTR,#TITLE		; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				ACALL	RX_BYTE			; Get byte from PC
				CJNE	A,#'R',CHK_MENU_1 ; Check 'R' receive?
				AJMP	DATA_READ		; Jump to read data

				ACALL	RX_BYTE			; Get byte from PC
CHK_MENU_1:		CJNE	A,#'S',MENU_MAIN ; Check 'S' receive?

				MOV		DPTR,#LOGGING	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port

				MOV		DPTR,#0000H		; Set ERAM Data pointer at 0000H
				MOV		R0,#060			; Set 60 records

LOOP:			ACALL	DS1820_RST		; DS1820 Reset
				ACALL	DS1820_PRES		; DS1820 Presence
				MOV		ONEWIRE_DATA,#0CCH ; Write Skip ROM
				ACALL	DS1820_WR		;
				MOV		ONEWIRE_DATA,#044H ; Write Convert Command
				ACALL	DS1820_WR		;

; (you can connect DS1820 as parasitic power by using this method)
				SETB	ONEWIRE			; Set onewire
				ACALL	DELAY_1s		; Delay 1 sec

				ACALL	DS1820_RST		; DS1820 Reset
				ACALL	DS1820_PRES		; DS1820 Presence
				MOV		ONEWIRE_DATA,#0CCH ; Write Skip ROM
				ACALL	DS1820_WR
				MOV		ONEWIRE_DATA,#0BEH ; Write Read Scratchpad Command
				ACALL	DS1820_WR

				ACALL	DS1820_RD		; Read DS1820
				MOV		TEMP,ONEWIRE_DATA ; Get First Byte as TEMP (L)

				ACALL	DS1820_RST		; DS1820 Reset
				ACALL	DS1820_PRES		; DS1820 Presence

				MOV		A,TEMP			; Get temperature data
				MOVX	@DPTR,A			; Store data in ERAM
				INC		DPTR			; Increase ERAM pointer

				PUSH	DPH				; Push ERAM DPTR
				PUSH	DPL				;
								
				MOV		DPTR,#LOGGING	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				MOV		A,R0            ; Get record remaind value
				DEC		ACC				; 
				ACALL	TX_DEC          ; Send record remain value
                
                POP		DPL				; Pop ERAM DPTR
                POP		DPH				;
                
				DJNZ	R0,LOOP         ; Loop until 60 record kept

				MOV		DPTR,#LOGGED	; Set Pointer Serial TX
				ACALL	TX_TEXT			; TX Text to Serial Port
				AJMP	MENU_MAIN		; Jump to main
			

DATA_READ:		MOV		DPTR,#0000H		; Set ERAM Data pointer at 0000H
				MOV		R0,#010			; Set display 10 lines
				
DATA_READ_LOOP:	MOV		R1,#006			; Set 6 records per line
DATA_READ_LOOP1: MOVX	A,@DPTR			; Get temperature data from ERAM
				INC		DPTR			; Increase ERAM pointer
				MOV		TEMP,A			; Send temperature to terminal
				ACALL	TX_TEMP         ;
				MOV		A,#' '			; Send space seperate
				ACALL	TX_ACC			;
				DJNZ	R1,DATA_READ_LOOP1 ; Loop until 6 records
				MOV		A,#00AH			; If yes, set terminal to new line
				ACALL	TX_ACC			; (line feed & carriage return)
				MOV		A,#00DH			;
				ACALL	TX_ACC			;
				DJNZ	R0,DATA_READ_LOOP ; Loop until 10 lines
				AJMP	MENU_MAIN		; Jump to main

;----------------------------------------------------------------------------
; RX byte from serial
;----------------------------------------------------------------------------
RX_BYTE:		JNB		RI,$			; Wait byte received
				CLR		RI				;
				MOV		A,SBUF			; Get byte from SBUF
				ACALL	UPCASE_ACC		; Upcase ACC
				RET						; Return

;----------------------------------------------------------------------------
; Upcase ACC.
;----------------------------------------------------------------------------
UPCASE_ACC:		CJNE	A,#07BH,UPCASE_ACC_1	; Check data valid in small case?
UPCASE_ACC_1:	JNC		UPCASE_EXIT				; If data => 07BH then exit
				CJNE	A,#061H,UPCASE_ACC_2	;
UPCASE_ACC_2:	JC		UPCASE_EXIT             ; or data < 061H then exit
				SUBB	A,#020H					; Else ubtract with 020H to UPCASE data
UPCASE_EXIT:	RET								; Return

;----------------------------------------------------------------------------
; TX temperature
;----------------------------------------------------------------------------
TX_TEMP:		MOV		A,TEMP			; Get Temp. Data
				CLR		C				; Clear Carry Flag
				RRC		A				; Rotate ACC. to Right with Carry

				MOV		F0,C			; Keep 0.5 degree bit

				MOV		B,#10			; Set divisor 10
				DIV		AB				; Divide A by 10
				ADD		A,#030H			; Convert result to ASCII (assume data valid on 0-9)
				CJNE	A,#'0',TX_NEXT1 ; Check tens digit = 0 ? 
				MOV		A,#' '			; If yes, send space instead of
TX_NEXT1:		ACALL	TX_ACC			; Send tens digit
				MOV		A,B             ; Get remainder
				ADD		A,#030H			; Convert result to ASCII
				ACALL	TX_ACC			; Send digit
				MOV		A,#'.'			; Send dot
				ACALL	TX_ACC			;
				MOV		C,F0			; Get half digit bit value
				JC		TX_5			; Check bit
				MOV		A,#'0'			; Send 0 if bit clear
				AJMP	TX_LAST			;
TX_5:			MOV		A,#'5'			; Send 5 if bit set
TX_LAST:		ACALL	TX_ACC			;
				MOV		A,#' '			; Send space seperate
				ACALL	TX_ACC			; 
				MOV		A,#'C'          ; Send C as celcius degree
				ACALL	TX_ACC			;
				RET						; Return to main

;----------------------------------------------------------------------------
; TX Decimal
;----------------------------------------------------------------------------
TX_DEC:			MOV		B,#10			; Set divisor 10
				DIV		AB				; Divide A by 10
				ADD		A,#030H			; Convert result to ASCII (assume data valid on 0-9)
				CJNE	A,#'0',TX_DEC1	; Check tens digit = 0 ? 
				MOV		A,#' '			; If yes, send space instead of
TX_DEC1:		ACALL	TX_ACC			; Send tens digit
				MOV		A,B             ; Get remainder
				ADD		A,#030H			; Convert result to ASCII
				ACALL	TX_ACC			; Send digit
				MOV		A,#' '			; Send space sperate
				ACALL	TX_ACC			;
				RET						; Return to main

;----------------------------------------------------------------------------
; TX ACC
;----------------------------------------------------------------------------
TX_ACC:			MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				RET						; Return

;----------------------------------------------------------------------------
; TX Serial Text from ROM Pointer
;----------------------------------------------------------------------------
TX_TEXT:		CLR		TI				; Clear TI
TX_LOOP:		CLR		A				; Clear ACC.
				MOVC	A,@A+DPTR		; Get Data from ROM with Pointer
				INC		DPTR			; Increase Pointer
				CJNE	A,#0FFH,TX_CHAR ; Check 0FFH End of Text Char.
				RET						; End => Return

TX_CHAR:		MOV		SBUF,A			; Send Data to SBUF
				JNB		TI,$			; Wait until TX already (TI=1)
				CLR		TI				; Clear TI
				AJMP	TX_LOOP			; Jump to TX_LOOP

;----------------------------------------------------------------------------
; DS1820 Data Read
;----------------------------------------------------------------------------
DS1820_RD:		MOV		R4,#8			; Set loop 8 times
				CLR		A				; Clear ACC.
DS1820_RD_LOOP: CLR		ONEWIRE			; Clear ONEWIRE
				NOP						; Delay
				NOP						;

				; P89C51RD2 addition
				NOP
				NOP
				; End

				SETB	ONEWIRE			; Set ONEWIRE
				NOP						; Delay
				NOP						;
				NOP						;
				NOP						;

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				MOV		C,ONEWIRE		; Get ONEWIRE to Carry Flag
				ACALL	ONEWIRE_DELAY	; Delay 75 us
				RRC		A		;
				DJNZ	R4,DS1820_RD_LOOP ; Do until 8 times
				MOV		ONEWIRE_DATA,A	; Move ACC. to ONEWIRE_DATA
				RET						; Return

;----------------------------------------------------------------------------
; DS1820 Data Write
;----------------------------------------------------------------------------
DS1820_WR:		MOV		R4,#8			; Set loop 8 times
				MOV		A,ONEWIRE_DATA	; Get ONEWIRE_DATA
DS1820_WR_LOOP: RRC		A				; Rotate ACC. with Carry Flag
				JNC		DS1820_WR_L		; Carry Flag was set ?
				CLR		ONEWIRE			; Set => TX high
				NOP						; Delay
				NOP						;
				NOP						;
				NOP						;

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				SETB	ONEWIRE			; Set ONEWIRE
				ACALL	ONEWIRE_DELAY	; Delay 75 us
				AJMP	DS1820_WR_NX	; Jump to next write
DS1820_WR_L:	CLR		ONEWIRE			; Clear => TX Low
				ACALL	ONEWIRE_DELAY	; Delay 75 us
				SETB	ONEWIRE			; Set ONEWIRE
				NOP						; Delay
				NOP						;
				NOP						;
				NOP						;

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

DS1820_WR_NX:	DJNZ	R4,DS1820_WR_LOOP ; Do until 8 times
				RET						; Return

;----------------------------------------------------------------------------
; DS1820 Reset
;----------------------------------------------------------------------------
DS1820_RST:		CLR		ONEWIRE			; Clear ONEWIRE
				ACALL	DELAY_1ms		; Delay
				SETB	ONEWIRE			; Set ONEWIRE
				MOV		R4,#16			; Delay
				DJNZ	R4,$			;
				RET						; Return

;----------------------------------------------------------------------------
; DS1820 Receive Presence Pulse
;----------------------------------------------------------------------------
DS1820_PRES:	MOV		R4,#16			; Set Loop wait 1
DS1820_PRES_1:	MOV		R3,#0			; Set Loop wait 2
DS1820_PRES_2:	JNB		ONEWIRE,DS1820_PRES_3 ; Check ONEWIRE was clear?
				DJNZ	R3,DS1820_PRES_2 ; Wait loop check 2
				DJNZ	R4,DS1820_PRES_1 ; Wait loop check 1
				RET						; Return
DS1820_PRES_3:	JNB		ONEWIRE,$		; Wait until ONEWIRE set
				MOV		R4,#16			; Delay
				DJNZ	R4,$			;
				CLR		BUSY			; Clear BUSY Flag
				RET						; Return

;----------------------------------------------------------------------------
; Dummy Delay time ONEWIRE_DELAY, 1ms, 10m, 1s
;----------------------------------------------------------------------------
ONEWIRE_DELAY:	MOV		R6,#012H		; Each loop = 75 us
ONEWIRE_DELAY_1: NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,ONEWIRE_DELAY_1
				RET

DELAY_1ms:		MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_1ms_1:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_1ms_1
				RET
				
DELAY_10ms:		MOV		R7,#010			; Do 10 times
DELAY_10ms_1:	MOV		R6,#0E6H		; Each loop = 1 ms
DELAY_10ms_2:	NOP
				NOP

				; P89C51RD2 addition
				NOP
				NOP
				NOP
				NOP
				; End

				DJNZ	R6,DELAY_10ms_2
				DJNZ	R7,DELAY_10ms_1
				RET

DELAY_1s:		MOV		R5,#100			; Do 100 times
DELAY_1s_1:		ACALL	DELAY_10ms
				DJNZ	R5,DELAY_1s_1
				RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
TITLE:			DB		00AH,00DH
				DB		00AH,00DH
				DB		'Room temperature data logger',00AH,00DH
				DB		'Press "S" to start 60 records logging',00AH,00DH
				DB		'or press "R" to read 60 logged data',00AH,00DH
				DB		00AH,00DH,0FFH

LOGGING:		DB		00DH,'Data logging remain : ',0FFH

LOGGED:			DB		00AH,00DH,'60 records logged',00AH,00DH,0FFH

READ_DATA:		DB		00AH,00DH,'60 records was read',00AH,00DH,0FFH,0FFH
